@@ -1,14 +1,17 @@
 {- adjusted branch
  -
- - Copyright 2016 Joey Hess <id@joeyh.name>
+ - Copyright 2016-2018 Joey Hess <id@joeyh.name>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 {-# LANGUAGE BangPatterns #-}
 
 module Annex.AdjustedBranch (
 	Adjustment(..),
+	LinkAdjustment(..),
+	PresenceAdjustment(..),
+	adjustmentHidesFiles,
 	OrigBranch,
 	AdjBranch(..),
 	originalToAdjusted,
@@ -16,17 +19,21 @@ module Annex.AdjustedBranch (
 	fromAdjustedBranch,
 	getAdjustment,
 	enterAdjustedBranch,
+	updateAdjustedBranch,
 	adjustBranch,
 	adjustToCrippledFileSystem,
-	updateAdjustedBranch,
+	mergeToAdjustedBranch,
 	propigateAdjustedCommits,
 	AdjustedClone(..),
 	checkAdjustedClone,
-	isGitVersionSupported,
+	isSupported,
 	checkVersionSupported,
+	isGitVersionSupported,
 ) where
 
 import Annex.Common
+import Types.AdjustedBranch
+import Annex.AdjustedBranch.Name
 import qualified Annex
 import Git
 import Git.Types
@@ -57,37 +64,33 @@ import Config
 
 import qualified Data.Map as M
 
-data Adjustment
-	= UnlockAdjustment
-	| LockAdjustment
-	| FixAdjustment
-	| UnFixAdjustment
-	| HideMissingAdjustment
-	| ShowMissingAdjustment
-	deriving (Show, Eq)
+-- How to perform various adjustments to a TreeItem.
+class AdjustTreeItem t where
+	adjustTreeItem :: t -> TreeItem -> Annex (Maybe TreeItem)
 
-reverseAdjustment :: Adjustment -> Adjustment
-reverseAdjustment UnlockAdjustment = LockAdjustment
-reverseAdjustment LockAdjustment = UnlockAdjustment
-reverseAdjustment HideMissingAdjustment = ShowMissingAdjustment
-reverseAdjustment ShowMissingAdjustment = HideMissingAdjustment
-reverseAdjustment FixAdjustment = UnFixAdjustment
-reverseAdjustment UnFixAdjustment = FixAdjustment
+instance AdjustTreeItem Adjustment where
+	adjustTreeItem (LinkAdjustment l) t = adjustTreeItem l t
+	adjustTreeItem (PresenceAdjustment p Nothing) t = adjustTreeItem p t
+	adjustTreeItem (PresenceAdjustment p (Just l)) t =
+		adjustTreeItem p t >>= \case
+			Nothing -> return Nothing
+			Just t' -> adjustTreeItem l t'
 
-{- How to perform various adjustments to a TreeItem. -}
-adjustTreeItem :: Adjustment -> TreeItem -> Annex (Maybe TreeItem)
-adjustTreeItem UnlockAdjustment = ifSymlink adjustToPointer noAdjust
-adjustTreeItem LockAdjustment = ifSymlink noAdjust adjustToSymlink
-adjustTreeItem FixAdjustment = ifSymlink adjustToSymlink noAdjust
-adjustTreeItem UnFixAdjustment = ifSymlink (adjustToSymlink' gitAnnexLinkCanonical) noAdjust
-adjustTreeItem HideMissingAdjustment = \ti@(TreeItem _ _ s) ->
-	catKey s >>= \case
-		Just k -> ifM (inAnnex k)
-			( return (Just ti)
-			, return Nothing
-			)
-		Nothing -> return (Just ti)
-adjustTreeItem ShowMissingAdjustment = noAdjust
+instance AdjustTreeItem LinkAdjustment where
+	adjustTreeItem UnlockAdjustment = ifSymlink adjustToPointer noAdjust
+	adjustTreeItem LockAdjustment = ifSymlink noAdjust adjustToSymlink
+	adjustTreeItem FixAdjustment = ifSymlink adjustToSymlink noAdjust
+	adjustTreeItem UnFixAdjustment = ifSymlink (adjustToSymlink' gitAnnexLinkCanonical) noAdjust
+
+instance AdjustTreeItem PresenceAdjustment where
+	adjustTreeItem HideMissingAdjustment = \ti@(TreeItem _ _ s) ->
+		catKey s >>= \case
+			Just k -> ifM (inAnnex k)
+				( return (Just ti)
+				, return Nothing
+				)
+			Nothing -> return (Just ti)
+	adjustTreeItem ShowMissingAdjustment = noAdjust
 
 ifSymlink :: (TreeItem -> Annex a) -> (TreeItem -> Annex a) -> TreeItem -> Annex a
 ifSymlink issymlink notsymlink ti@(TreeItem _f m _s)
@@ -118,9 +121,6 @@ adjustToSymlink' gitannexlink ti@(TreeItem f _m s) = catKey s >>= \case
 			<$> hashSymlink linktarget
 	Nothing -> return (Just ti)
 
-type OrigBranch = Branch
-newtype AdjBranch = AdjBranch { adjBranch :: Branch }
-
 -- This is a hidden branch ref, that's used as the basis for the AdjBranch,
 -- since pushes can overwrite the OrigBranch at any time. So, changes
 -- are propigated from the AdjBranch to the head of the BasisBranch.
@@ -132,42 +132,6 @@ basisBranch :: AdjBranch -> BasisBranch
 basisBranch (AdjBranch adjbranch) = BasisBranch $
 	Ref ("refs/basis/" ++ fromRef (Git.Ref.base adjbranch))
 
-adjustedBranchPrefix :: String
-adjustedBranchPrefix = "refs/heads/adjusted/"
-
-serialize :: Adjustment -> String
-serialize UnlockAdjustment = "unlocked"
-serialize LockAdjustment = "locked"
-serialize HideMissingAdjustment = "present"
-serialize ShowMissingAdjustment = "showmissing"
-serialize FixAdjustment = "fixed"
-serialize UnFixAdjustment = "unfixed"
-
-deserialize :: String -> Maybe Adjustment
-deserialize "unlocked" = Just UnlockAdjustment
-deserialize "locked" = Just UnlockAdjustment
-deserialize "present" = Just HideMissingAdjustment
-deserialize "fixed" = Just FixAdjustment
-deserialize "unfixed" = Just UnFixAdjustment
-deserialize _ = Nothing
-
-originalToAdjusted :: OrigBranch -> Adjustment -> AdjBranch
-originalToAdjusted orig adj = AdjBranch $ Ref $
-	adjustedBranchPrefix ++ base ++ '(' : serialize adj ++ ")"
-  where
-	base = fromRef (Git.Ref.base orig)
-
-adjustedToOriginal :: Branch -> Maybe (Adjustment, OrigBranch)
-adjustedToOriginal b
-	| adjustedBranchPrefix `isPrefixOf` bs = do
-		let (base, as) = separate (== '(') (drop prefixlen bs)
-		adj <- deserialize (takeWhile (/= ')') as)
-		Just (adj, Git.Ref.branchRef (Ref base))
-	| otherwise = Nothing
-  where
-	bs = fromRef b
-	prefixlen = length adjustedBranchPrefix
-
 getAdjustment :: Branch -> Maybe Adjustment
 getAdjustment = fmap fst . adjustedToOriginal
 
@@ -182,12 +146,21 @@ originalBranch = fmap fromAdjustedBranch <$> inRepo Git.Branch.current
  - branch).
  -
  - Can fail, if no branch is checked out, or if the adjusted branch already
- - exists, or perhaps if staged changes conflict with the adjusted branch.
+ - exists, or if staged changes prevent a checkout.
  -}
 enterAdjustedBranch :: Adjustment -> Annex Bool
-enterAdjustedBranch adj = go =<< originalBranch
+enterAdjustedBranch adj = inRepo Git.Branch.current >>= \case
+	Just currbranch -> case getAdjustment currbranch of
+		Just curradj | curradj == adj ->
+			updateAdjustedBranch adj (AdjBranch currbranch)
+				(fromAdjustedBranch currbranch)
+		_ -> go currbranch
+	Nothing -> do
+		warning "not on any branch!"
+		return False
   where
-	go (Just origbranch) = do
+	go currbranch = do
+		let origbranch = fromAdjustedBranch currbranch
 		let adjbranch = adjBranch $ originalToAdjusted origbranch adj
 		ifM (inRepo (Git.Ref.exists adjbranch) <&&> (not <$> Annex.getState Annex.force))
 			( do
@@ -200,26 +173,62 @@ enterAdjustedBranch adj = go =<< originalBranch
 					  , Git.Ref.describe origbranch
 					  ]
 					, [ "You can check out the adjusted branch manually to enter it,"
-					  , "or delete the adjusted branch and re-run this command."
+					  , "or add the --force option to overwrite the old branch."
 					  ]
 					]
 				return False
 			, do
-				AdjBranch b <- preventCommits $ const $ 
+				b <- preventCommits $ const $ 
 					adjustBranch adj origbranch
-				showOutput -- checkout can have output in large repos
-				inRepo $ Git.Command.runBool
-					[ Param "checkout"
-					, Param $ fromRef $ Git.Ref.base b
-					]
+				checkoutAdjustedBranch b []
 			)
-	go Nothing = do
-		warning "not on any branch!"
-		return False
+
+checkoutAdjustedBranch :: AdjBranch -> [CommandParam] -> Annex Bool
+checkoutAdjustedBranch (AdjBranch b) checkoutparams = do
+	showOutput -- checkout can have output in large repos
+	inRepo $ Git.Command.runBool $
+		[ Param "checkout"
+		, Param $ fromRef $ Git.Ref.base b
+		-- always show checkout progress, even if --quiet is used
+		-- to suppress other messages
+		, Param "--progress"
+		] ++ checkoutparams
+
+{- Already in a branch with this adjustment, but the user asked to enter it
+ - again. This should have the same result as checking out the original branch,
+ - deleting and rebuilding the adjusted branch, and then checking it out.
+ - But, it can be implemented more efficiently than that.
+ -}
+updateAdjustedBranch :: Adjustment -> AdjBranch -> OrigBranch -> Annex Bool
+updateAdjustedBranch adj@(PresenceAdjustment _ _) (AdjBranch currbranch) origbranch = do
+	b <- preventCommits $ \commitlck -> do
+		-- Avoid losing any commits that the adjusted branch has that
+		-- have not yet been propigated back to the origbranch.
+		_ <- propigateAdjustedCommits' origbranch adj commitlck
+
+		-- Git normally won't do anything when asked to check out the
+		-- currently checked out branch, even when its ref has
+		-- changed. Work around this by writing a raw sha to .git/HEAD.
+		inRepo (Git.Ref.sha currbranch) >>= \case
+			Just headsha -> inRepo $ \r ->
+				writeFile (Git.Ref.headFile r) (fromRef headsha)
+			_ -> noop
+	
+		adjustBranch adj origbranch
+	
+	-- Make git checkout quiet to avoid warnings about disconnected
+	-- branch tips being lost.
+	checkoutAdjustedBranch b [Param "--quiet"]
+updateAdjustedBranch adj@(LinkAdjustment _) _ origbranch = preventCommits $ \commitlck -> do
+	-- Not really needed here, but done for consistency.
+	_ <- propigateAdjustedCommits' origbranch adj commitlck
+	-- No need to do anything else, because link adjustments are stable.
+	return True
 
 adjustToCrippledFileSystem :: Annex ()
 adjustToCrippledFileSystem = do
 	warning "Entering an adjusted branch where files are unlocked as this filesystem does not support locked files."
+	checkVersionSupported
 	whenM (isNothing <$> originalBranch) $
 		void $ inRepo $ Git.Branch.commitCommand Git.Branch.AutomaticCommit
 			[ Param "--quiet"
@@ -227,7 +236,7 @@ adjustToCrippledFileSystem = do
 			, Param "-m"
 			, Param "commit before entering adjusted unlocked branch"
 			]
-	unlessM (enterAdjustedBranch UnlockAdjustment) $
+	unlessM (enterAdjustedBranch (LinkAdjustment UnlockAdjustment)) $
 		warning "Failed to enter adjusted branch!"
 
 setBasisBranch :: BasisBranch -> Ref -> Annex ()
@@ -313,8 +322,8 @@ findAdjustingCommit (AdjBranch b) = go =<< catCommit b
 {- Update the currently checked out adjusted branch, merging the provided
  - branch into it. Note that the provided branch should be a non-adjusted
  - branch. -}
-updateAdjustedBranch :: Branch -> (OrigBranch, Adjustment) -> [Git.Merge.MergeConfig] -> Annex Bool -> Git.Branch.CommitMode -> Annex Bool
-updateAdjustedBranch tomerge (origbranch, adj) mergeconfig canresolvemerge commitmode = catchBoolIO $
+mergeToAdjustedBranch :: Branch -> (OrigBranch, Adjustment) -> [Git.Merge.MergeConfig] -> Annex Bool -> Git.Branch.CommitMode -> Annex Bool
+mergeToAdjustedBranch tomerge (origbranch, adj) mergeconfig canresolvemerge commitmode = catchBoolIO $
 	join $ preventCommits go
   where
 	adjbranch@(AdjBranch currbranch) = originalToAdjusted origbranch adj
@@ -603,10 +612,8 @@ checkAdjustedClone = ifM isBareRepo
 				, return NeedUpgradeForAdjustedClone
 				)
 
--- git 2.2.0 needed for GIT_COMMON_DIR which is needed
--- by updateAdjustedBranch to use withWorkTreeRelated.
-isGitVersionSupported :: IO Bool
-isGitVersionSupported = not <$> Git.Version.older "2.2.0"
+isSupported :: Annex Bool
+isSupported = versionSupportsAdjustedBranch <&&> liftIO isGitVersionSupported
 
 checkVersionSupported :: Annex ()
 checkVersionSupported = do
@@ -614,3 +621,8 @@ checkVersionSupported = do
 		giveup "Adjusted branches are only supported in v6 or newer repositories."
 	unlessM (liftIO isGitVersionSupported) $
 		giveup "Your version of git is too old; upgrade it to 2.2.0 or newer to use adjusted branches."
+
+-- git 2.2.0 needed for GIT_COMMON_DIR which is needed
+-- by updateAdjustedBranch to use withWorkTreeRelated.
+isGitVersionSupported :: IO Bool
+isGitVersionSupported = not <$> Git.Version.older "2.2.0"
