@@ -12,6 +12,7 @@ module Annex.Content (
 	inAnnex',
 	inAnnexSafe,
 	inAnnexCheck,
+	objectFileExists,
 	lockContentShared,
 	lockContentForRemoval,
 	ContentRemovalLock,
@@ -46,6 +47,7 @@ module Annex.Content (
 	staleKeysPrune,
 	pruneTmpWorkDirBefore,
 	isUnmodified,
+	isUnmodifiedCheap,
 	verifyKeyContent,
 	VerifyConfig(..),
 	Verification(..),
@@ -97,29 +99,29 @@ inAnnex key = inAnnexCheck key $ liftIO . doesFileExist
 inAnnexCheck :: Key -> (FilePath -> Annex Bool) -> Annex Bool
 inAnnexCheck key check = inAnnex' id False check key
 
-{- inAnnex that performs an arbitrary check of the key's content.
- -
- - When the content is unlocked, it must also be unmodified, or the bad
- - value will be returned.
- -
- - In direct mode, at least one of the associated files must pass the
- - check. Additionally, the file must be unmodified.
- -}
+{- inAnnex that performs an arbitrary check of the key's content. -}
 inAnnex' :: (a -> Bool) -> a -> (FilePath -> Annex a) -> Key -> Annex a
 inAnnex' isgood bad check key = withObjectLoc key checkindirect checkdirect
   where
 	checkindirect loc = do
 		r <- check loc
 		if isgood r
-			then do
-				cache <- Database.Keys.getInodeCaches key
-				if null cache
-					then return r
-					else ifM (sameInodeCache loc cache)
-						( return r
-						, return bad
-						)
+			then ifM (annexThin <$> Annex.getGitConfig)
+				-- When annex.thin is set, the object file
+				-- could be modified; make sure it's not.
+				-- (Suppress any messages about
+				-- checksumming, to avoid them cluttering
+				-- the display.)
+				( ifM (doQuietAction $ isUnmodified key loc)
+					( return r
+					, return bad
+					)
+				, return r
+				)
 			else return bad
+ 
+	-- In direct mode, at least one of the associated files must pass the
+	-- check. Additionally, the file must be unmodified.
 	checkdirect [] = return bad
 	checkdirect (loc:locs) = do
 		r <- check loc
@@ -130,6 +132,11 @@ inAnnex' isgood bad check key = withObjectLoc key checkindirect checkdirect
 				)
 			else checkdirect locs
 
+{- Like inAnnex, checks if the object file for a key exists,
+ - but there are no guarantees it has the right content. -}
+objectFileExists :: Key -> Annex Bool
+objectFileExists key = calcRepo (gitAnnexLocation key) >>= liftIO . doesFileExist
+
 {- A safer check; the key's content must not only be present, but
  - is not in the process of being removed. -}
 inAnnexSafe :: Key -> Annex (Maybe Bool)
@@ -219,6 +226,8 @@ lockContentShared key a = lockContentUsing lock key $ ifM (inAnnex key)
 
 {- Exclusively locks content, while performing an action that
  - might remove it.
+ -
+ - If locking fails, throws an exception rather than running the action.
  -}
 lockContentForRemoval :: Key -> (ContentRemovalLock -> Annex a) -> Annex a
 lockContentForRemoval key a = lockContentUsing lock key $ 
@@ -746,17 +755,38 @@ isUnmodified :: Key -> FilePath -> Annex Bool
 isUnmodified key f = go =<< geti
   where
 	go Nothing = return False
-	go (Just fc) = cheapcheck fc <||> expensivecheck fc
-	cheapcheck fc = anyM (compareInodeCaches fc)
-		=<< Database.Keys.getInodeCaches key
+	go (Just fc) = isUnmodifiedCheap' key fc <||> expensivecheck fc
 	expensivecheck fc = ifM (verifyKeyContent RetrievalAllKeysSecure AlwaysVerify UnVerified key f)
-		-- The file could have been modified while it was
-		-- being verified. Detect that.
-		( geti >>= maybe (return False) (compareInodeCaches fc)
+		( do
+			-- The file could have been modified while it was
+			-- being verified. Detect that.
+			ifM (geti >>= maybe (return False) (compareInodeCaches fc))
+				( do
+					-- Update the InodeCache to avoid
+					-- performing this expensive check again.
+					Database.Keys.addInodeCaches key [fc]
+					return True
+				, return False
+				)
 		, return False
 		)
 	geti = withTSDelta (liftIO . genInodeCache f)
 
+{- Cheap check if a file contains the unmodified content of the key,
+ - only checking the InodeCache of the key.
+ -
+ - Note that, on systems not supporting high-resolution mtimes,
+ - this may report a false positive when repeated edits are made to a file
+ - within a small time window (eg 1 second).
+ -}
+isUnmodifiedCheap :: Key -> FilePath -> Annex Bool
+isUnmodifiedCheap key f = maybe (return False) (isUnmodifiedCheap' key) 
+	=<< withTSDelta (liftIO . genInodeCache f)
+
+isUnmodifiedCheap' :: Key -> InodeCache -> Annex Bool
+isUnmodifiedCheap' key fc = 
+	anyM (compareInodeCaches fc) =<< Database.Keys.getInodeCaches key
+
 {- Moves a key out of .git/annex/objects/ into .git/annex/bad, and
  - returns the file it was moved to. -}
 moveBad :: Key -> Annex FilePath
@@ -845,6 +875,7 @@ getKeysPresent keyloc = do
 saveState :: Bool -> Annex ()
 saveState nocommit = doSideAction $ do
 	Annex.Queue.flush
+	Database.Keys.closeDb
 	unless nocommit $
 		whenM (annexAlwaysCommit <$> Annex.getGitConfig) $
 			Annex.Branch.commit =<< Annex.Branch.commitMessage
