@@ -5,8 +5,6 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
-{-# LANGUAGE CPP #-}
-
 module Annex.Environment where
 
 import Annex.Common
@@ -37,14 +35,8 @@ checkEnvironmentIO = whenM (isNothing <$> myUserGecos) $ do
 	ensureEnv "GIT_AUTHOR_NAME" username
 	ensureEnv "GIT_COMMITTER_NAME" username
   where
-#ifndef __ANDROID__
 	-- existing environment is not overwritten
 	ensureEnv var val = setEnv var val False
-#else
-	-- Environment setting is broken on Android, so this is dealt with
-	-- in runshell instead.
-	ensureEnv _ _ = noop
-#endif
 
 {- Runs an action that commits to the repository, and if it fails, 
  - sets user.email and user.name to a dummy value and tries the action again. -}
