@@ -32,13 +32,12 @@ import Annex.Content.Direct
 import Annex.Perms
 import Annex.Link
 import Annex.MetaData
+import Annex.CurrentBranch
 import Annex.Version
 import Logs.Location
 import qualified Annex
 import qualified Annex.Queue
 import qualified Database.Keys
-import qualified Git
-import qualified Git.Branch
 import Config
 import Utility.InodeCache
 import Annex.ReplaceFile
@@ -288,7 +287,7 @@ makeLink file key mcache = flip catchNonAsync (restoreFile file key) $ do
 	-- touch symlink to have same time as the original file,
 	-- as provided in the InodeCache
 	case mcache of
-		Just c -> liftIO $ touch file (TimeSpec $ inodeCacheToMtime c) False
+		Just c -> liftIO $ touch file (inodeCacheToMtime c) False
 		Nothing -> noop
 
 	return l
@@ -329,21 +328,13 @@ addUnlocked = isDirect <||>
 	(versionSupportsUnlockedPointers <&&>
 	 ((not . coreSymlinks <$> Annex.getGitConfig) <||>
 	  (annexAddUnlocked <$> Annex.getGitConfig) <||>
-	  (maybe False (\b -> getAdjustment b == Just UnlockAdjustment) <$> cachedCurrentBranch)
+	  (maybe False isadjustedunlocked . snd <$> getCurrentBranch)
 	 )
 	)
-
-cachedCurrentBranch :: Annex (Maybe Git.Branch)
-cachedCurrentBranch = maybe cache (return . Just)
-	=<< Annex.getState Annex.cachedcurrentbranch
   where
-	cache :: Annex (Maybe Git.Branch)
-	cache = inRepo Git.Branch.currentUnsafe >>= \case
-		Nothing -> return Nothing
-		Just b -> do
-			Annex.changeState $ \s ->
-				s { Annex.cachedcurrentbranch = Just b }
-			return (Just b)
+	isadjustedunlocked (LinkAdjustment UnlockAdjustment) = True
+	isadjustedunlocked (PresenceAdjustment _ (Just UnlockAdjustment)) = True
+	isadjustedunlocked _ = False
 
 {- Adds a file to the work tree for the key, and stages it in the index.
  - The content of the key may be provided in a temp file, which will be
