@@ -21,14 +21,15 @@ module Annex.Init (
 import Annex.Common
 import qualified Annex
 import qualified Git
-import qualified Git.LsFiles
 import qualified Git.Config
 import qualified Git.Objects
+import qualified Git.LsFiles
 import qualified Annex.Branch
 import Logs.UUID
 import Logs.Trust.Basic
 import Logs.Config
 import Types.TrustLevel
+import Types.RepoVersion
 import Annex.Version
 import Annex.Difference
 import Annex.UUID
@@ -37,7 +38,7 @@ import Annex.WorkTree
 import Config
 import Config.Smudge
 import Annex.Direct
-import Annex.AdjustedBranch
+import qualified Annex.AdjustedBranch as AdjustedBranch
 import Annex.Environment
 import Annex.Hook
 import Annex.InodeSentinal
@@ -78,7 +79,7 @@ genDescription Nothing = do
 		Right username -> [username, at, hostname, ":", reldir]
 		Left _ -> [hostname, ":", reldir]
 
-initialize :: AutoInit -> Maybe String -> Maybe Version -> Annex ()
+initialize :: AutoInit -> Maybe String -> Maybe RepoVersion -> Annex ()
 initialize ai mdescription mversion = checkCanInitialize ai $ do
 	{- Has to come before any commits are made as the shared
 	 - clone heuristic expects no local objects. -}
@@ -98,7 +99,7 @@ initialize ai mdescription mversion = checkCanInitialize ai $ do
 
 -- Everything except for uuid setup, shared clone setup, and initial
 -- description.
-initialize' :: AutoInit -> Maybe Version -> Annex ()
+initialize' :: AutoInit -> Maybe RepoVersion -> Annex ()
 initialize' ai mversion = checkCanInitialize ai $ do
 	checkLockSupport
 	checkFifoSupport
@@ -112,18 +113,16 @@ initialize' ai mversion = checkCanInitialize ai $ do
 	whenM versionSupportsUnlockedPointers $ do
 		configureSmudgeFilter
 		scanUnlockedFiles
-	checkAdjustedClone >>= \case
-		NeedUpgradeForAdjustedClone -> 
-			void $ upgrade True  versionForAdjustedClone
-		InAdjustedClone -> return ()
-		NotInAdjustedClone ->
+		unlessM isBareRepo $ do
+			hookWrite postCheckoutHook
+			hookWrite postMergeHook
+	AdjustedBranch.checkAdjustedClone >>= \case
+		AdjustedBranch.NeedUpgradeForAdjustedClone -> 
+			void $ upgrade True versionForAdjustedClone
+		AdjustedBranch.InAdjustedClone -> return ()
+		AdjustedBranch.NotInAdjustedClone ->
 			ifM (crippledFileSystem <&&> (not <$> isBareRepo))
-				( ifM versionSupportsUnlockedPointers
-					( adjustToCrippledFileSystem
-					, do
-						enableDirectMode
-						setDirect True
-					)
+				( adjustToCrippledFilesystem
 				-- Handle case where this repo was cloned from a
 				-- direct mode repo
 				, unlessM isBareRepo
@@ -266,15 +265,6 @@ checkFifoSupport = unlessM probeFifoSupport $ do
 	warning "Disabling ssh connection caching."
 	setConfig (annexConfig "sshcaching") (Git.Config.boolConfig False)
 
-enableDirectMode :: Annex ()
-enableDirectMode = unlessM isDirect $ do
-	warning "Enabling direct mode."
-	top <- fromRepo Git.repoPath
-	(l, clean) <- inRepo $ Git.LsFiles.inRepo [top]
-	forM_ l $ \f ->
-		maybe noop (`toDirect` f) =<< isAnnexLink f
-	void $ liftIO clean
-
 checkSharedClone :: Annex Bool
 checkSharedClone = inRepo Git.Objects.isSharedClone
 
@@ -294,3 +284,20 @@ propigateSecureHashesOnly :: Annex ()
 propigateSecureHashesOnly =
 	maybe noop (setConfig (ConfigKey "annex.securehashesonly"))
 		=<< getGlobalConfig "annex.securehashesonly"
+
+adjustToCrippledFilesystem :: Annex ()
+adjustToCrippledFilesystem = ifM (liftIO $ AdjustedBranch.isGitVersionSupported)
+	( do
+		void $ upgrade True versionForCrippledFilesystem
+		AdjustedBranch.adjustToCrippledFileSystem
+	, enableDirectMode 
+	)
+
+enableDirectMode :: Annex ()
+enableDirectMode = unlessM isDirect $ do
+	warning "Enabling direct mode."
+	top <- fromRepo Git.repoPath
+	(l, clean) <- inRepo $ Git.LsFiles.inRepo [top]
+	forM_ l $ \f ->
+		maybe noop (`toDirect` f) =<< isAnnexLink f
+	void $ liftIO clean
