@@ -10,6 +10,7 @@ module Annex.MetaData.StandardFields (
 	yearMetaField,
 	monthMetaField,
 	dayMetaField,
+	isDateMetaField,
 	lastChangedField,
 	mkLastChangedField,
 	isLastChangedField
@@ -31,6 +32,13 @@ monthMetaField = mkMetaFieldUnchecked "month"
 dayMetaField :: MetaField
 dayMetaField = mkMetaFieldUnchecked "day"
 
+isDateMetaField :: MetaField -> Bool
+isDateMetaField f
+	| f == yearMetaField = True
+	| f == monthMetaField = True
+	| f == dayMetaField = True
+	| otherwise = False
+
 lastChangedField :: MetaField
 lastChangedField = mkMetaFieldUnchecked lastchanged
 
