@@ -25,10 +25,10 @@ import Data.Ord
 findExisting :: RemoteName -> Annex (Maybe (UUID, RemoteConfig))
 findExisting name = do
 	t <- trustMap
-	matches <- sortBy (comparing $ \(u, _c) -> M.lookup u t)
+	headMaybe
+		. sortBy (comparing $ \(u, _c) -> Down $ M.lookup u t)
 		. findByName name
 		<$> Logs.Remote.readRemoteLog
-	return $ headMaybe matches
 
 newConfig :: RemoteName -> RemoteConfig
 newConfig = M.singleton nameKey
