@@ -20,7 +20,6 @@ import qualified Annex
 import qualified Git
 import qualified Git.Config
 import qualified Git.Command
-import qualified Command.Sync
 import Config.Files
 import Utility.FreeDesktop
 import Utility.DiskFree
@@ -30,6 +29,7 @@ import Utility.Mounts
 import Utility.DataUnits
 import Remote (prettyUUID)
 import Annex.UUID
+import Annex.CurrentBranch
 import Types.StandardGroups
 import Logs.PreferredContent
 import Logs.UUID
@@ -165,13 +165,8 @@ postFirstRepositoryR :: Handler Html
 postFirstRepositoryR = page "Getting started" (Just Configuration) $ do
 	unlessM (liftIO $ inPath "git") $
 		giveup "You need to install git in order to use git-annex!"
-#ifdef __ANDROID__
-	androidspecial <- liftIO $ doesDirectoryExist "/sdcard/DCIM"
-	let path = "/sdcard/annex"
-#else
 	androidspecial <- liftIO osAndroid
 	path <- liftIO . defaultRepositoryPath =<< liftH inFirstRun
-#endif
 	((res, form), enctype) <- liftH $ runFormPostNoToken $ newRepositoryForm path
 	case res of
 		FormSuccess (RepositoryPath p) -> liftH $
@@ -180,12 +175,8 @@ postFirstRepositoryR = page "Getting started" (Just Configuration) $ do
 
 getAndroidCameraRepositoryR :: Handler ()
 getAndroidCameraRepositoryR = do
-#ifdef __ANDROID__
-	let dcim = "/sdcard/DCIM"
-#else
 	home <- liftIO myHomeDir
 	let dcim = home </> "storage" </> "dcim"
-#endif
 	startFullAssistant dcim SourceGroup $ Just addignore	
   where
 	addignore = do
@@ -221,7 +212,7 @@ postNewRepositoryR = page "Add another repository" (Just Configuration) $ do
  - immediately pulling from it. Also spawns a sync to push to it as well. -}
 immediateSyncRemote :: Remote -> Assistant ()
 immediateSyncRemote r = do
-	currentbranch <- liftAnnex $ join Command.Sync.getCurrBranch
+	currentbranch <- liftAnnex $ getCurrentBranch
 	void $ manualPull currentbranch [r]
 	syncRemote r
 
@@ -392,10 +383,6 @@ driveList = mapM (genRemovableDrive . mnt_dir) =<< filter sane <$> getMounts
 		| dir == "/tmp" = False
 		| dir == "/run/shm" = False
 		| dir == "/run/lock" = False
-#ifdef __ANDROID__
-		| dir == "/mnt/sdcard" = False
-		| dir == "/sdcard" = False
-#endif
 		| otherwise = True
 #endif
 
