@@ -13,7 +13,6 @@ import Options.Applicative
 #if ! MIN_VERSION_optparse_applicative(0,14,1)
 import Options.Applicative.Builder.Internal
 #endif
-import Control.Concurrent
 import qualified Data.Map as M
 
 import Annex.Common
@@ -175,10 +174,7 @@ data KeyOptions
 
 parseKeyOptions :: Parser KeyOptions
 parseKeyOptions = parseAllOption
-	<|> WantBranchKeys <$> some (option (str >>= pure . Ref)
-		( long "branch" <> metavar paramRef
-		<> help "operate on files in the specified branch or treeish"
-		))
+	<|> parseBranchKeysOption
 	<|> flag' WantUnusedKeys
 		( long "unused" <> short 'U'
 		<> help "operate on files found by last run of git-annex unused"
@@ -188,6 +184,13 @@ parseKeyOptions = parseAllOption
 		<> help "operate on specified key"
 		))
 
+parseBranchKeysOption :: Parser KeyOptions
+parseBranchKeysOption =
+	WantBranchKeys <$> some (option (str >>= pure . Ref)
+		( long "branch" <> metavar paramRef
+		<> help "operate on files in the specified branch or treeish"
+		))
+
 parseFailedTransfersOption :: Parser KeyOptions
 parseFailedTransfersOption = flag' WantFailedTransfers
 	( long "failed"
@@ -212,18 +215,18 @@ parseKey = maybe (fail "invalid key") return . file2key
 -- Options to match properties of annexed files.
 annexedMatchingOptions :: [GlobalOption]
 annexedMatchingOptions = concat
-	[ nonWorkTreeMatchingOptions'
+	[ keyMatchingOptions'
 	, fileMatchingOptions'
 	, combiningOptions
 	, timeLimitOption
 	]
 
--- Matching options that don't need to examine work tree files.
-nonWorkTreeMatchingOptions :: [GlobalOption]
-nonWorkTreeMatchingOptions = nonWorkTreeMatchingOptions' ++ combiningOptions
+-- Matching options that can operate on keys as well as files.
+keyMatchingOptions :: [GlobalOption]
+keyMatchingOptions = keyMatchingOptions' ++ combiningOptions ++ timeLimitOption
 
-nonWorkTreeMatchingOptions' :: [GlobalOption]
-nonWorkTreeMatchingOptions' = 
+keyMatchingOptions' :: [GlobalOption]
+keyMatchingOptions' = 
 	[ globalSetter Limit.addIn $ strOption
 		( long "in" <> short 'i' <> metavar paramRemote
 		<> help "match files present in a remote"
@@ -286,7 +289,7 @@ nonWorkTreeMatchingOptions' =
 
 -- Options to match files which may not yet be annexed.
 fileMatchingOptions :: [GlobalOption]
-fileMatchingOptions = fileMatchingOptions' ++ combiningOptions
+fileMatchingOptions = fileMatchingOptions' ++ combiningOptions ++ timeLimitOption
 
 fileMatchingOptions' :: [GlobalOption]
 fileMatchingOptions' =
@@ -370,11 +373,7 @@ jobsOption =
 			)
 	]
   where
-	set n = do
-		Annex.changeState $ \s -> s { Annex.concurrency = Concurrent n }
-		c <- liftIO getNumCapabilities
-		when (n > c) $
-			liftIO $ setNumCapabilities n
+	set n = Annex.changeState $ \s -> s { Annex.concurrency = Concurrent n }
 
 timeLimitOption :: [GlobalOption]
 timeLimitOption = 
