@@ -97,7 +97,7 @@ seek :: AddUrlOptions -> CommandSeek
 seek o = allowConcurrentOutput $ do
 	forM_ (addUrls o) (\u -> go (o, u))
 	case batchOption o of
-		Batch -> batchInput (parseBatchInput o) go
+		Batch fmt -> batchInput fmt (parseBatchInput o) go
 		NoBatch -> noop
   where
 	go (o', u) = do
@@ -132,17 +132,21 @@ checkUrl r o u = do
 	go deffile (Right (UrlContents sz mf)) = do
 		let f = adjustFile o (fromMaybe (maybe deffile fromSafeFilePath mf) (fileOption (downloadOptions o)))
 		void $ commandAction $ startRemote r o f u sz
-	go deffile (Right (UrlMulti l))
-		| isNothing (fileOption (downloadOptions o)) =
+	go deffile (Right (UrlMulti l)) = case fileOption (downloadOptions o) of
+		Nothing ->
 			forM_ l $ \(u', sz, f) -> do
 				let f' = adjustFile o (deffile </> fromSafeFilePath f)
-				void $ commandAction $
-					startRemote r o f' u' sz
-		| otherwise = giveup $ unwords
-			[ "That url contains multiple files according to the"
-			, Remote.name r
-			, " remote; cannot add it to a single file."
-			]
+				void $ commandAction $ startRemote r o f' u' sz
+		Just f -> case l of
+			[] -> noop
+			((u',sz,_):[]) -> do
+				let f' = adjustFile o f
+				void $ commandAction $ startRemote r o f' u' sz
+			_ -> giveup $ unwords
+				[ "That url contains multiple files according to the"
+				, Remote.name r
+				, " remote; cannot add it to a single file."
+				]
 
 startRemote :: Remote -> AddUrlOptions -> FilePath -> URLString -> Maybe Integer -> CommandStart
 startRemote r o file uri sz = do
@@ -233,7 +237,8 @@ addUrlChecked o url file u checkexistssize key =
 			(exists, samesize, url') <- checkexistssize key
 			if exists && (samesize || relaxedOption (downloadOptions o))
 				then do
-					setUrlPresent u key url'
+					setUrlPresent key url'
+					logChange key u InfoPresent
 					next $ return True
 				else do
 					warning $ "while adding a new url to an already annexed file, " ++ if exists
@@ -397,7 +402,8 @@ addWorkTree u url file key mtmp = case mtmp of
   where
 	go = do
 		maybeShowJSON $ JSONChunk [("key", key2file key)]
-		setUrlPresent u key url
+		setUrlPresent key url
+		logChange key u InfoPresent
 		ifM (addAnnexedFile file key mtmp)
 			( do
 				when (isJust mtmp) $
