@@ -36,7 +36,7 @@ cmd = noCommit $ dontCheck repoExists $
 		"uid" (withParams seek)
 
 seek :: CmdParams -> CommandSeek
-seek = withWords start
+seek = withWords (commandAction . start)
 
 -- This runs as root, so avoid making any commits or initializing
 -- git-annex, or doing other things that create root-owned files.
@@ -122,6 +122,7 @@ checkHiddenService = bracket setup cleanup go
 			, connCheckAuth = const False
 			, connIhdl = h
 			, connOhdl = h
+			, connIdent = ConnIdent Nothing
 			}
 		runst <- mkRunState Client
 		void $ runNetProto runst conn $ P2P.serveAuth u
