@@ -99,10 +99,12 @@ changeExport r ea db new = do
 	-- the next block of code below may have renamed some files to
 	-- temp files. Diff from the incomplete tree to the new tree,
 	-- and delete any temp files that the new tree can't use.
+	let recover diff = commandAction $
+		startRecoverIncomplete r ea db
+			(Git.DiffTree.srcsha diff)
+			(Git.DiffTree.file diff)
 	forM_ (concatMap incompleteExportedTreeish old) $ \incomplete ->
-		mapdiff (\diff -> startRecoverIncomplete r ea db (Git.DiffTree.srcsha diff) (Git.DiffTree.file diff))
-			incomplete
-			new
+		mapdiff recover incomplete new
 
 	-- Diff the old and new trees, and delete or rename to new name all
 	-- changed files in the export. After this, every file that remains
@@ -111,17 +113,18 @@ changeExport r ea db new = do
 	-- When there was an export conflict, this resolves it.
 	--
 	-- The ExportTree is also updated here to reflect the new tree.
-	case map exportedTreeish old of
+	case nub (map exportedTreeish old) of
 		[] -> updateExportTree db emptyTree new
 		[oldtreesha] -> do
 			diffmap <- mkDiffMap oldtreesha new db
-			let seekdiffmap a = seekActions $ pure $ map a (M.toList diffmap)
+			let seekdiffmap a = commandActions $ 
+				map a (M.toList diffmap)
 			-- Rename old files to temp, or delete.
 			seekdiffmap $ \(ek, (moldf, mnewf)) -> do
 				case (moldf, mnewf) of
 					(Just oldf, Just _newf) ->
 						startMoveToTempName r ea db oldf ek
-					(Just oldf, Nothing) -> 
+					(Just oldf, Nothing) ->
 						startUnexport' r ea db oldf ek
 					_ -> stop
 			-- Rename from temp to new files.
@@ -144,7 +147,7 @@ changeExport r ea db new = do
 				-- Don't rename to temp, because the
 				-- content is unknown; delete instead.
 				mapdiff
-					(\diff -> startUnexport r ea db (Git.DiffTree.file diff) (unexportboth diff))
+					(\diff -> commandAction $ startUnexport r ea db (Git.DiffTree.file diff) (unexportboth diff))
 					oldtreesha new
 			updateExportTree db emptyTree new
 	liftIO $ recordExportTreeCurrent db new
@@ -194,22 +197,26 @@ fillExport :: Remote -> ExportActions Annex -> ExportHandle -> Git.Ref -> Annex
 fillExport r ea db new = do
 	(l, cleanup) <- inRepo $ Git.LsTree.lsTree new
 	cvar <- liftIO $ newMVar False
-	seekActions $ pure $ map (startExport r ea db cvar) l
+	commandActions $ map (startExport r ea db cvar) l
 	void $ liftIO $ cleanup
 	liftIO $ takeMVar cvar
 
 startExport :: Remote -> ExportActions Annex -> ExportHandle -> MVar Bool -> Git.LsTree.TreeItem -> CommandStart
 startExport r ea db cvar ti = do
 	ek <- exportKey (Git.LsTree.sha ti)
-	stopUnless (notpresent ek) $ do
+	stopUnless (notrecordedpresent ek) $ do
 		showStart ("export " ++ name r) f
-		liftIO $ modifyMVar_ cvar (pure . const True)
-		next $ performExport r ea db ek af (Git.LsTree.sha ti) loc
+		ifM (either (const False) id <$> tryNonAsync (checkPresentExport ea (asKey ek) loc))
+			( next $ next $ cleanupExport r db ek loc False
+			, do
+				liftIO $ modifyMVar_ cvar (pure . const True)
+				next $ performExport r ea db ek af (Git.LsTree.sha ti) loc
+			)
   where
 	loc = mkExportLocation f
 	f = getTopFilePath (Git.LsTree.file ti)
 	af = AssociatedFile (Just f)
-	notpresent ek = (||)
+	notrecordedpresent ek = (||)
 		<$> liftIO (notElem loc <$> getExportedLocation db (asKey ek))
 		-- If content was removed from the remote, the export db
 		-- will still list it, so also check location tracking.
@@ -242,13 +249,14 @@ performExport r ea db ek af contentsha loc = do
 				liftIO $ hClose h
 				storer tmp sha1k loc m
 	if sent
-		then next $ cleanupExport r db ek loc
+		then next $ cleanupExport r db ek loc True
 		else stop
 
-cleanupExport :: Remote -> ExportHandle -> ExportKey -> ExportLocation -> CommandCleanup
-cleanupExport r db ek loc = do
+cleanupExport :: Remote -> ExportHandle -> ExportKey -> ExportLocation -> Bool -> CommandCleanup
+cleanupExport r db ek loc sent = do
 	liftIO $ addExportedLocation db (asKey ek) loc
-	logChange (asKey ek) (uuid r) InfoPresent
+	when sent $
+		logChange (asKey ek) (uuid r) InfoPresent
 	return True
 
 startUnexport :: Remote -> ExportActions Annex -> ExportHandle -> TopFilePath -> [Git.Sha] -> CommandStart
