@@ -18,8 +18,10 @@ import Annex.Content
 import Annex.Perms
 import qualified Annex.Queue
 import qualified Database.Keys
-#if ! defined(mingw32_HOST_OS) && ! defined(__ANDROID__)
+
+#if ! defined(mingw32_HOST_OS)
 import Utility.Touch
+import System.Posix.Files
 #endif
 
 cmd :: Command
@@ -34,8 +36,9 @@ seek ps = unlessM crippledFileSystem $ do
 		( return FixAll
 		, return FixSymlinks
 		)
-	l <- workTreeItems ps
-	flip withFilesInGit l $ whenAnnexed $ start fixwhat
+	withFilesInGit
+		(commandAction . (whenAnnexed $ start fixwhat))
+		=<< workTreeItems ps
 
 data FixWhat = FixSymlinks | FixAll
 
@@ -90,15 +93,15 @@ makeHardLink file key = do
 fixSymlink :: FilePath -> FilePath -> CommandPerform
 fixSymlink file link = do
 	liftIO $ do
-#if ! defined(mingw32_HOST_OS) && ! defined(__ANDROID__)
+#if ! defined(mingw32_HOST_OS)
 		-- preserve mtime of symlink
-		mtime <- catchMaybeIO $ TimeSpec . modificationTime
+		mtime <- catchMaybeIO $ modificationTimeHiRes
 			<$> getSymbolicLinkStatus file
 #endif
 		createDirectoryIfMissing True (parentDir file)
 		removeFile file
 		createSymbolicLink link file
-#if ! defined(mingw32_HOST_OS) && ! defined(__ANDROID__)
+#if ! defined(mingw32_HOST_OS)
 		maybe noop (\t -> touch file t False) mtime
 #endif
 	next $ cleanupSymlink file
