@@ -18,7 +18,7 @@ cmd = noMessages $ command "groupwanted" SectionSetup
 	(withParams seek)
 
 seek :: CmdParams -> CommandSeek
-seek = withWords start
+seek = withWords (commandAction . start)
 
 start :: [String] -> CommandStart
 start (g:[]) = next $ performGet groupPreferredContentMapRaw g
