@@ -13,6 +13,7 @@ import Command
 import Config
 import qualified Command.Add
 import qualified Command.Fix
+import qualified Command.Smudge
 import Annex.Direct
 import Annex.Hook
 import Annex.Link
@@ -39,7 +40,7 @@ seek :: CmdParams -> CommandSeek
 seek ps = lockPreCommitHook $ ifM isDirect
 	( do
 		-- update direct mode mappings for committed files
-		withWords startDirect ps
+		withWords (commandAction . startDirect) ps
 		runAnnexHook preCommitAnnexHook
 	, do
 		ifM (not <$> versionSupportsUnlockedPointers <&&> liftIO Git.haveFalseIndex)
@@ -51,14 +52,24 @@ seek ps = lockPreCommitHook $ ifM isDirect
 			, do
 				l <- workTreeItems ps
 				-- fix symlinks to files being committed
-				flip withFilesToBeCommitted l $ \f -> 
+				flip withFilesToBeCommitted l $ \f -> commandAction $
 					maybe stop (Command.Fix.start Command.Fix.FixSymlinks f)
 						=<< isAnnexLink f
-				-- inject unlocked files into the annex
-				-- (not needed when repo version uses
-				-- unlocked pointer files)
-				unlessM versionSupportsUnlockedPointers $
-					withFilesOldUnlockedToBeCommitted startInjectUnlocked l
+				ifM versionSupportsUnlockedPointers
+					-- after a merge conflict or git
+					-- cherry-pick or stash, pointer
+					-- files in the worktree won't
+					-- be populated, so populate them
+					-- here
+					( Command.Smudge.updateSmudged 
+						-- When there's a false index,
+						-- restaging the files won't work.
+						. Restage =<< liftIO Git.haveFalseIndex
+					-- inject unlocked files into the annex
+					-- (not needed when repo version uses
+					-- unlocked pointer files)
+					, withFilesOldUnlockedToBeCommitted (commandAction . startInjectUnlocked) l
+					)
 			)
 		runAnnexHook preCommitAnnexHook
 		-- committing changes to a view updates metadata
