@@ -49,8 +49,8 @@ batchParser s = case separate (== ' ') (reverse s) of
 
 seek :: ReKeyOptions -> CommandSeek
 seek o = case batchOption o of
-	Batch -> batchInput batchParser (batchCommandAction . start)
-	NoBatch -> withPairs (start . parsekey) (reKeyThese o)
+	Batch fmt -> batchInput fmt batchParser (batchCommandAction . start)
+	NoBatch -> withPairs (commandAction . start . parsekey) (reKeyThese o)
   where
 	parsekey (file, skey) =
 		(file, fromMaybe (giveup "bad key") (file2key skey))
@@ -68,7 +68,7 @@ perform :: FilePath -> Key -> Key -> CommandPerform
 perform file oldkey newkey = do
 	ifM (inAnnex oldkey) 
 		( unlessM (linkKey file oldkey newkey) $
-			giveup "failed"
+			giveup "failed creating link from old to new key"
 		, unlessM (Annex.getState Annex.force) $
 			giveup $ file ++ " is not available (use --force to override)"
 		)
@@ -87,18 +87,18 @@ linkKey file oldkey newkey = ifM (isJust <$> isAnnexLink file)
 		oldobj <- calcRepo (gitAnnexLocation oldkey)
 		isJust <$> linkOrCopy' (return True) newkey oldobj tmp Nothing
 	, do
-		ic <- withTSDelta (liftIO . genInodeCache file)
-	 	{- The file being rekeyed is itself an unlocked file, so if
-		 - it's linked to the old key, that link must be broken. -}
+	 	{- The file being rekeyed is itself an unlocked file; if
+		 - it's hard linked to the old key, that link must be broken. -}
 		oldobj <- calcRepo (gitAnnexLocation oldkey)
-		v <- tryNonAsync $ modifyContent oldobj $ do
-			replaceFile oldobj $ \tmp ->
-				unlessM (checkedCopyFile oldkey file tmp Nothing) $
-					error "can't lock old key"
-			freezeContent oldobj
-			oldic <- withTSDelta (liftIO . genInodeCache oldobj)
-			whenM (isUnmodified oldkey oldobj) $
-				Database.Keys.addInodeCaches oldkey (catMaybes [oldic])
+		v <- tryNonAsync $ do
+			st <- liftIO $ getFileStatus file
+			when (linkCount st > 1) $ do
+				freezeContent oldobj
+				replaceFile file $ \tmp -> do
+					unlessM (checkedCopyFile oldkey oldobj tmp Nothing) $
+						error "can't lock old key"
+					thawContent tmp
+		ic <- withTSDelta (liftIO . genInodeCache file)
 		case v of
 			Left e -> do
 				warning (show e)
