@@ -22,7 +22,7 @@ cmd = noCommit $ dontCheck repoExists $
 		paramNothing (withParams seek)
 
 seek :: CmdParams -> CommandSeek
-seek = withNothing start
+seek = withNothing (commandAction start)
 
 start :: CommandStart
 start = next $ next $ runRepair =<< Annex.getState Annex.force
