@@ -19,7 +19,7 @@ cmd = command "resolvemerge" SectionPlumbing
 	paramNothing (withParams seek)
 
 seek :: CmdParams -> CommandSeek
-seek = withNothing start
+seek = withNothing (commandAction start)
 
 start :: CommandStart
 start = do
