@@ -9,7 +9,6 @@
 module Command.Sync (
 	cmd,
 	CurrBranch,
-	getCurrBranch,
 	mergeConfig,
 	merge,
 	prepMerge,
@@ -20,7 +19,7 @@ module Command.Sync (
 	pushBranch,
 	updateBranch,
 	syncBranch,
-	updateSyncBranch,
+	updateBranches,
 	seekExportContent,
 ) where
 
@@ -60,6 +59,7 @@ import Annex.UpdateInstead
 import Annex.Export
 import Annex.LockFile
 import Annex.TaggedPush
+import Annex.CurrentBranch
 import qualified Database.Export as Export
 import Utility.Bloom
 import Utility.OptParse
@@ -162,8 +162,7 @@ seek :: SyncOptions -> CommandSeek
 seek o = allowConcurrentOutput $ do
 	prepMerge
 
-	getbranch <- getCurrBranch 
-	let withbranch a = a =<< getbranch
+	let withbranch a = a =<< getCurrentBranch
 
 	remotes <- syncRemotes (syncWith o)
 	let gitremotes = filter Remote.gitSyncableRemote remotes
@@ -188,8 +187,8 @@ seek o = allowConcurrentOutput $ do
 				]
 			
 			whenM shouldsynccontent $ do
-				syncedcontent <- seekSyncContent o dataremotes
-				exportedcontent <- seekExportContent exportremotes
+				syncedcontent <- withbranch $ seekSyncContent o dataremotes
+				exportedcontent <- withbranch $ seekExportContent exportremotes
 				-- Transferring content can take a while,
 				-- and other changes can be pushed to the
 				-- git-annex branch on the remotes in the
@@ -209,35 +208,6 @@ seek o = allowConcurrentOutput $ do
 		<||> pure (not (null (contentOfOption o)))
 		<||> (pure (not (noContentOption o)) <&&> getGitConfigVal annexSyncContent)
 
-type CurrBranch = (Maybe Git.Branch, Maybe Adjustment)
-
-{- There may not be a branch checked out until after the commit,
- - or perhaps after it gets merged from the remote, or perhaps
- - never.
- -
- - So only look it up once it's needed, and once there is a
- - branch, cache it.
- -
- - When on an adjusted branch, gets the original branch, and the adjustment.
- -}
-getCurrBranch :: Annex (Annex CurrBranch)
-getCurrBranch = do
-	mvar <- liftIO newEmptyMVar
-	return $ ifM (liftIO $ isEmptyMVar mvar)
-		( do
-			currbranch <- inRepo Git.Branch.current
-			case currbranch of
-				Nothing -> return (Nothing, Nothing)
-				Just b -> do
-					let v = case adjustedToOriginal b of
-						Nothing -> (Just b, Nothing)
-						Just (adj, origbranch) ->
-							(Just origbranch, Just adj)
-					liftIO $ putMVar mvar v
-					return v
-		, liftIO $ readMVar mvar
-		)
-
 {- Merging may delete the current directory, so go to the top
  - of the repo. This also means that sync always acts on all files in the
  - repository, not just on a subdirectory. -}
@@ -257,7 +227,7 @@ mergeConfig =
 
 merge :: CurrBranch -> [Git.Merge.MergeConfig] -> ResolveMergeOverride -> Git.Branch.CommitMode -> Git.Branch -> Annex Bool
 merge currbranch mergeconfig resolvemergeoverride commitmode tomerge = case currbranch of
-	(Just b, Just adj) -> updateAdjustedBranch tomerge (b, adj) mergeconfig canresolvemerge commitmode
+	(Just b, Just adj) -> mergeToAdjustedBranch tomerge (b, adj) mergeconfig canresolvemerge commitmode
 	(b, _) -> autoMergeFrom tomerge b mergeconfig canresolvemerge commitmode
   where
 	canresolvemerge = case resolvemergeoverride of
@@ -372,17 +342,29 @@ needMerge (Just branch, madj) = ifM (allM id checks)
 
 pushLocal :: CurrBranch -> CommandStart
 pushLocal b = do
-	updateSyncBranch b
+	updateBranches b
 	stop
 
-updateSyncBranch :: CurrBranch -> Annex ()
-updateSyncBranch (Nothing, _) = noop
-updateSyncBranch (Just branch, madj) = do
+updateBranches :: CurrBranch -> Annex ()
+updateBranches (Nothing, _) = noop
+updateBranches (Just branch, madj) = do
 	-- When in an adjusted branch, propigate any changes made to it
-	-- back to the original branch.
-	maybe noop (propigateAdjustedCommits branch) madj
+	-- back to the original branch. The adjusted branch may also need
+	-- to be updated to hide/expose files.
+	case madj of
+		Nothing -> noop
+		Just adj -> do
+			let origbranch = branch
+			propigateAdjustedCommits origbranch adj
+			when (adjustmentHidesFiles adj) $ do
+				showSideAction "updating adjusted branch"
+				let adjbranch = originalToAdjusted origbranch adj
+				unlessM (updateAdjustedBranch adj adjbranch origbranch) $
+					warning $ unwords [ "Updating adjusted branch failed." ]
+					
 	-- Update the sync branch to match the new state of the branch
 	inRepo $ updateBranch (syncBranch branch) branch
+
 	-- In direct mode, we're operating on some special direct mode
 	-- branch, rather than the intended branch, so update the intended
 	-- branch.
@@ -568,8 +550,11 @@ newer remote b = do
 		, return True
 		)
 
-{- Without --all, only looks at files in the work tree. With --all,
- - makes 2 passes, first looking at the work tree and then all keys.
+{- Without --all, only looks at files in the work tree.
+ - (Or, when in an ajusted branch where some files are hidden, at files in
+ - the original branch.)
+ -
+ - With --all, makes a second pass over all keys.
  - This ensures that preferred content expressions that match on
  - filenames work, even when in --all mode.
  -
@@ -577,25 +562,42 @@ newer remote b = do
  -
  - When concurrency is enabled, files are processed concurrently.
  -}
-seekSyncContent :: SyncOptions -> [Remote] -> Annex Bool
-seekSyncContent o rs = do
+seekSyncContent :: SyncOptions -> [Remote] -> CurrBranch -> Annex Bool
+seekSyncContent o rs currbranch = do
 	mvar <- liftIO newEmptyMVar
 	bloom <- case keyOptions o of
 		Just WantAllKeys -> Just <$> genBloomFilter (seekworktree mvar [])
-		_ -> do
-			l <- workTreeItems (contentOfOption o)
-			seekworktree mvar l (const noop)
-			pure Nothing
+		_ -> case currbranch of
+                	(Just origbranch, Just adj) | adjustmentHidesFiles adj -> do
+				l <- workTreeItems' (AllowHidden True) (contentOfOption o)
+				seekincludinghidden origbranch mvar l (const noop)
+				pure Nothing
+			_ -> do
+				l <- workTreeItems (contentOfOption o)
+				seekworktree mvar l (const noop)
+				pure Nothing
 	withKeyOptions' (keyOptions o) False
-		(return (seekkeys mvar bloom))
+		(return (gokey mvar bloom))
 		(const noop)
 		[]
 	finishCommandActions
 	liftIO $ not <$> isEmptyMVar mvar
   where
-	seekworktree mvar l bloomfeeder = seekHelper LsFiles.inRepo l >>=
-		mapM_ (\f -> ifAnnexed f (go (Right bloomfeeder) mvar (AssociatedFile (Just f))) noop)
-	seekkeys mvar bloom k _ = go (Left bloom) mvar (AssociatedFile Nothing) k
+	seekworktree mvar l bloomfeeder = 
+		seekHelper LsFiles.inRepo l
+			>>= gofiles bloomfeeder mvar
+
+	seekincludinghidden origbranch mvar l bloomfeeder = 
+		seekHelper (LsFiles.inRepoOrBranch origbranch) l 
+			>>= gofiles bloomfeeder mvar
+
+	gofiles bloomfeeder mvar = mapM_ $ \f ->
+		ifAnnexed f
+			(go (Right bloomfeeder) mvar (AssociatedFile (Just f)))
+			noop
+	
+	gokey mvar bloom (k, _) = go (Left bloom) mvar (AssociatedFile Nothing) k
+
 	go ebloom mvar af k = commandAction $ do
 		whenM (syncFile ebloom rs af k) $
 			void $ liftIO $ tryPutMVar mvar ()
@@ -680,30 +682,45 @@ syncFile ebloom rs af k = onlyActionOn' k $ do
  - 
  - Returns True if any file transfers were made.
  -}
-seekExportContent :: [Remote] -> Annex Bool
-seekExportContent rs = or <$> forM rs go
+seekExportContent :: [Remote] -> CurrBranch -> Annex Bool
+seekExportContent rs (currbranch, _) = or <$> forM rs go
   where
 	go r = withExclusiveLock (gitAnnexExportLock (Remote.uuid r)) $ do
 		db <- Export.openDb (Remote.uuid r)
 		ea <- Remote.exportActions r
 		exported <- case remoteAnnexExportTracking (Remote.gitconfig r) of
-			Nothing -> getExport (Remote.uuid r)
+			Nothing -> nontracking r
 			Just b -> do
 				mcur <- inRepo $ Git.Ref.tree b
 				case mcur of
-					Nothing -> getExport (Remote.uuid r)
+					Nothing -> nontracking r
 					Just cur -> do
 						Command.Export.changeExport r ea db cur
 						return [Exported cur []]
 		Export.closeDb db `after` fillexport r ea db exported
+		
+	nontracking r = do
+		exported <- getExport (Remote.uuid r)
+		maybe noop (warnnontracking r exported) currbranch
+		return exported
+	
+	warnnontracking r exported currb = inRepo (Git.Ref.tree currb) >>= \case
+		Just currt | not (any (\ex -> exportedTreeish ex == currt) exported) ->
+			showLongNote $ unwords
+				[ "Not updating export to " ++ Remote.name r
+				, "to reflect changes to the tree, because export"
+				, "tracking is not enabled. "
+				, "(Use git-annex export's --tracking option"
+				, "to enable it.)"
+				]
+		_ -> noop
+
 
 	fillexport _ _ _ [] = return False
 	fillexport r ea db (Exported { exportedTreeish = t }:[]) =
 		Command.Export.fillExport r ea db t
 	fillexport r _ _ _ = do
-		warning $ "Export conflict detected. Different trees have been exported to " ++ 
-			Remote.name r ++ 
-			". Use git-annex export to resolve this conflict."
+		warnExportConflict r
 		return False
 
 cleanupLocal :: CurrBranch -> CommandStart
