@@ -17,7 +17,7 @@ cmd = notBareRepo $ notDirect $
 		paramView (withParams seek)
 
 seek :: CmdParams -> CommandSeek
-seek = withWords start
+seek = withWords (commandAction . start)
 
 start :: [String] -> CommandStart
 start params = do
