@@ -254,7 +254,7 @@ tryGitConfigRead autoinit r
 		v <- liftIO $ withTmpFile "git-annex.tmp" $ \tmpfile h -> do
 			hClose h
 			let url = Git.repoLocation r ++ "/config"
-			ifM (Url.download nullMeterUpdate url tmpfile uo)
+			ifM (Url.downloadQuiet nullMeterUpdate url tmpfile uo)
 				( Just <$> pipedconfig "git" [Param "config", Param "--null", Param "--list", Param "--file", File tmpfile]
 				, return Nothing
 				)
@@ -377,7 +377,9 @@ keyUrls gc repo r key = map tourl locs'
 dropKey :: Remote -> State -> Key -> Annex Bool
 dropKey r st key = do
 	repo <- getRepo r
-	dropKey' repo r st key
+	catchNonAsync
+		(dropKey' repo r st key)
+		(\e -> warning (show e) >> return False)
 
 dropKey' :: Git.Repo -> Remote -> State -> Key -> Annex Bool
 dropKey' repo r (State connpool duc _) key
