@@ -17,6 +17,9 @@ import Backend
 import Remote.Helper.Encryptable (isEncrypted)
 import Database.Export
 import Annex.Export
+import Config
+import Git.Types (fromRef)
+import Logs.Export
 
 import qualified Data.Map as M
 import Control.Concurrent.STM
@@ -69,13 +72,16 @@ adjustExportableRemoteType rt = rt { setup = setup' }
 -- remote to be an export.
 adjustExportable :: Remote -> Annex Remote
 adjustExportable r = case M.lookup "exporttree" (config r) of
-	Just "yes" -> ifM (isExportSupported r)
-		( isexport
-		, notexport
-		)
 	Nothing -> notexport
-	Just "no" -> notexport
-	Just _ -> error "bad exporttree value"
+	Just c -> case yesNo c of
+		Just True -> ifM (isExportSupported r)
+			( isexport
+			, notexport
+			)
+		Just False -> notexport
+		Nothing -> do
+			warning $ "bad exporttree value for " ++ name r ++ ", assuming not an export"
+			notexport
   where
 	notexport = return $ r 
 		{ exportActions = exportUnsupported
@@ -85,23 +91,39 @@ adjustExportable r = case M.lookup "exporttree" (config r) of
 		}
 	isexport = do
 		db <- openDb (uuid r)
+		updateflag <- liftIO $ newTVarIO Nothing
 
-		updateflag <- liftIO newEmptyTMVarIO
-		let updateonce = liftIO $ atomically $
-			ifM (isEmptyTMVar updateflag)
-				( do
-					putTMVar updateflag ()
+		-- When multiple threads run this, all except the first
+		-- will block until the first runs doneupdateonce.
+		-- Returns True when an update should be done and False
+		-- when the update has already been done.
+		let startupdateonce = liftIO $ atomically $
+			readTVar updateflag >>= \case
+				Nothing -> do
+					writeTVar updateflag (Just True)
 					return True
-				, return False
-				)
+				Just True -> retry
+				Just False -> return False
+		let doneupdateonce = \updated ->
+			when updated $
+				liftIO $ atomically $
+					writeTVar updateflag (Just False)
 		
+		exportinconflict <- liftIO $ newTVarIO False
+
 		-- Get export locations for a key. Checks once
 		-- if the export log is different than the database and
 		-- updates the database, to notice when an export has been
 		-- updated from another repository.
 		let getexportlocs = \k -> do
-			whenM updateonce $
-				updateExportTreeFromLog db
+			bracket startupdateonce doneupdateonce $ \updatenow ->
+				when updatenow $
+					updateExportTreeFromLog db >>= \case
+						ExportUpdateSuccess -> return ()
+						ExportUpdateConflict -> do
+							warnExportConflict r
+							liftIO $ atomically $
+								writeTVar exportinconflict True
 			liftIO $ getExportTree db k
 
 		return $ r
@@ -123,7 +145,7 @@ adjustExportable r = case M.lookup "exporttree" (config r) of
 			-- so don't need to use retrieveExport.
 			, retrieveKeyFile = if appendonly r
 				then retrieveKeyFile r
-				else retrieveKeyFileFromExport getexportlocs
+				else retrieveKeyFileFromExport getexportlocs exportinconflict
 			, retrieveKeyFileCheap = if appendonly r
 				then retrieveKeyFileCheap r
 				else \_ _ _ -> return False
@@ -157,18 +179,30 @@ adjustExportable r = case M.lookup "exporttree" (config r) of
 					ea <- exportActions r
 					anyM (checkPresentExport ea k)
 						=<< getexportlocs k
+			-- checkPresent from an export is more expensive
+			-- than otherwise, so not cheap. Also, this
+			-- avoids things that look at checkPresentCheap and
+			-- silently skip non-present files from behaving
+			-- in confusing ways when there's an export
+			-- conflict.
+			, checkPresentCheap = False
 			, mkUnavailable = return Nothing
 			, getInfo = do
+				ts <- map (fromRef . exportedTreeish)
+					<$> getExport (uuid r)
 				is <- getInfo r
-				return (is++[("export", "yes")])
+				return (is++[("export", "yes"), ("exportedtree", unwords ts)])
 			}
-	retrieveKeyFileFromExport getexportlocs k _af dest p = unVerified $
+	retrieveKeyFileFromExport getexportlocs exportinconflict k _af dest p = unVerified $
 		if maybe False (isJust . verifyKeyContent) (maybeLookupBackendVariety (keyVariety k))
 			then do
 				locs <- getexportlocs k
 				case locs of
 					[] -> do
-						warning "unknown export location"
+						ifM (liftIO $ atomically $ readTVar exportinconflict)
+							( warning "unknown export location, likely due to the export conflict"
+							, warning "unknown export location"
+							)
 						return False
 					(l:_) -> do
 						ea <- exportActions r
