@@ -1,6 +1,6 @@
 {- git-annex test suite
  -
- - Copyright 2010-2017 Joey Hess <id@joeyh.name>
+ - Copyright 2010-2018 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU GPL version 3 or higher.
  -}
@@ -10,6 +10,7 @@
 module Test where
 
 import Types.Test
+import Types.RepoVersion
 import Test.Framework
 import Options.Applicative.Types
 
@@ -59,7 +60,6 @@ import qualified Annex.WorkTree
 import qualified Annex.Init
 import qualified Annex.CatFile
 import qualified Annex.Path
-import qualified Annex.AdjustedBranch
 import qualified Annex.VectorClock
 import qualified Annex.View
 import qualified Annex.View.ViewedFile
@@ -103,28 +103,25 @@ optParser = TestOptions
 		)
 	<*> cmdParams "non-options are for internal use only"
 
-runner :: Maybe (TestOptions -> IO ())
-runner = Just go
+runner :: TestOptions -> IO ()
+runner opts
+	| fakeSsh opts = runFakeSsh (internalData opts)
+	| otherwise = runsubprocesstests =<< Utility.Env.getEnv subenv
   where
-	go opts
-		| fakeSsh opts = runFakeSsh (internalData opts)
-		| otherwise = runsubprocesstests opts
-			=<< Utility.Env.getEnv subenv
-	
 	-- Run git-annex test in a subprocess, so that any files
 	-- it may open will be closed before running finalCleanup.
 	-- This should prevent most failures to clean up after the test
 	-- suite.
 	subenv = "GIT_ANNEX_TEST_SUBPROCESS"
-	runsubprocesstests opts Nothing = do
+	runsubprocesstests Nothing = do
 		pp <- Annex.Path.programPath
 		Utility.Env.Set.setEnv subenv "1" True
 		ps <- getArgs
-		(Nothing, Nothing, Nothing, pid) <-createProcess (proc pp ps)
+		(Nothing, Nothing, Nothing, pid) <- createProcess (proc pp ps)
 		exitcode <- waitForProcess pid
 		unless (keepFailuresOption opts) finalCleanup
 		exitWith exitcode
-	runsubprocesstests opts (Just _) = isolateGitConfig $ do
+	runsubprocesstests (Just _) = isolateGitConfig $ do
 		ensuretmpdir
 		crippledfilesystem <- fst <$> Annex.Init.probeCrippledFileSystem' tmpdir
 		case tryIngredients ingredients (tastyOptionSet opts) (tests crippledfilesystem opts) of
@@ -148,10 +145,10 @@ tests crippledfilesystem opts = testGroup "Tests" $ properties :
 	map (\(d, te) -> withTestMode te initTests (unitTests d)) testmodes
   where
 	testmodes = catMaybes
-		[ Just ("v6 unlocked", (testMode opts "6") { unlockedFiles = True })
-		, unlesscrippled ("v5", testMode opts "5")
-		, unlesscrippled ("v6 locked", testMode opts "6")
-		, Just ("v5 direct", (testMode opts "5") { forceDirect = True })
+		[ Just ("v7 unlocked", (testMode opts (RepoVersion 7)) { unlockedFiles = True })
+		, unlesscrippled ("v5", testMode opts (RepoVersion 5))
+		, unlesscrippled ("v7 locked", testMode opts (RepoVersion 7))
+		, Just ("v5 direct", (testMode opts (RepoVersion 5)) { forceDirect = True })
 		]
 	unlesscrippled v
 		| crippledfilesystem = Nothing
@@ -225,7 +222,7 @@ unitTests note = testGroup ("Unit Tests " ++ note)
 	, testCase "move (ssh remote)" test_move_ssh_remote
 	, testCase "copy" test_copy
 	, testCase "lock" test_lock
-	, testCase "lock (v6 --force)" test_lock_v6_force
+	, testCase "lock (v7 --force)" test_lock_v7_force
 	, testCase "edit (no pre-commit)" test_edit
 	, testCase "edit (pre-commit)" test_edit_precommit
 	, testCase "partial commit" test_partial_commit
@@ -280,7 +277,7 @@ test_init = innewrepo $ do
 	ver <- annexVersion <$> getTestMode
 	if ver == Annex.Version.defaultVersion
 		then git_annex "init" [reponame] @? "init failed"
-		else git_annex "init" [reponame, "--version", ver] @? "init failed"
+		else git_annex "init" [reponame, "--version", show (fromRepoVersion ver)] @? "init failed"
 	setupTestMode
   where
 	reponame = "test repo"
@@ -289,10 +286,10 @@ test_init = innewrepo $ do
 -- annexed file that later tests will use
 test_add :: Assertion
 test_add = inmainrepo $ do
-	writeFile annexedfile $ content annexedfile
+	writecontent annexedfile $ content annexedfile
 	add_annex annexedfile @? "add failed"
 	annexed_present annexedfile
-	writeFile sha1annexedfile $ content sha1annexedfile
+	writecontent sha1annexedfile $ content sha1annexedfile
 	git_annex "add" [sha1annexedfile, "--backend=SHA1"] @? "add with SHA1 failed"
 	whenM (unlockedFiles <$> getTestMode) $
 		git_annex "unlock" [sha1annexedfile] @? "unlock failed"
@@ -300,12 +297,12 @@ test_add = inmainrepo $ do
 	checkbackend sha1annexedfile backendSHA1
 	ifM (annexeval Config.isDirect)
 		( do
-			writeFile ingitfile $ content ingitfile
+			writecontent ingitfile $ content ingitfile
 			not <$> boolSystem "git" [Param "add", File ingitfile] @? "git add failed to fail in direct mode"
 			nukeFile ingitfile
 			git_annex "sync" [] @? "sync failed"
 		, do
-			writeFile ingitfile $ content ingitfile
+			writecontent ingitfile $ content ingitfile
 			boolSystem "git" [Param "add", File ingitfile] @? "git add failed"
 			boolSystem "git" [Param "commit", Param "-q", Param "-m", Param "commit"] @? "git commit failed"
 			git_annex "add" [ingitfile] @? "add ingitfile should be no-op"
@@ -314,14 +311,14 @@ test_add = inmainrepo $ do
 
 test_add_dup :: Assertion
 test_add_dup = intmpclonerepo $ do
-	writeFile annexedfiledup $ content annexedfiledup
+	writecontent annexedfiledup $ content annexedfiledup
 	add_annex annexedfiledup @? "add of second file with same content failed"
 	annexed_present annexedfiledup
 	annexed_present annexedfile
 
 test_add_extras :: Assertion
 test_add_extras = intmpclonerepo $ do
-	writeFile wormannexedfile $ content wormannexedfile
+	writecontent wormannexedfile $ content wormannexedfile
 	git_annex "add" [wormannexedfile, "--backend=WORM"] @? "add with WORM failed"
 	whenM (unlockedFiles <$> getTestMode) $
 		git_annex "unlock" [wormannexedfile] @? "unlock failed"
@@ -378,7 +375,7 @@ test_import = intmpclonerepo $ Utility.Tmp.Dir.withTmpDir "importtest" $ \import
 	git_annex "drop" ["--force", imported1, imported2, imported5] @? "drop failed"
 	annexed_notpresent_imported imported2
 	(toimportdup, importfdup, importeddup) <- mktoimport importdir "importdup"
-	not <$> git_annex "import" ["--clean-duplicates", toimportdup] 
+	git_annex_shouldfail "import" ["--clean-duplicates", toimportdup] 
 		@? "import of missing duplicate with --clean-duplicates failed to fail"
 	checkdoesnotexist importeddup
 	checkexists importfdup
@@ -386,7 +383,7 @@ test_import = intmpclonerepo $ Utility.Tmp.Dir.withTmpDir "importtest" $ \import
 	mktoimport importdir subdir = do
 		createDirectory (importdir </> subdir)
 		let importf = subdir </> "f"
-		writeFile (importdir </> importf) (content importf)
+		writecontent (importdir </> importf) (content importf)
 		return (importdir </> subdir, importdir </> importf, importf)
 	annexed_present_imported f = ifM (annexeval Config.crippledFileSystem)
 		( annexed_present_unlocked f
@@ -401,7 +398,7 @@ test_reinject :: Assertion
 test_reinject = intmpclonerepoInDirect $ do
 	git_annex "drop" ["--force", sha1annexedfile] @? "drop failed"
 	annexed_notpresent sha1annexedfile
-	writeFile tmp $ content sha1annexedfile
+	writecontent tmp $ content sha1annexedfile
 	key <- Key.key2file <$> getKey backendSHA1 tmp
 	git_annex "reinject" [tmp, sha1annexedfile] @? "reinject failed"
 	annexed_present sha1annexedfile
@@ -436,7 +433,7 @@ test_drop_noremote = intmpclonerepo $ do
 	git_annex "get" [annexedfile] @? "get failed"
 	boolSystem "git" [Param "remote", Param "rm", Param "origin"]
 		@? "git remote rm origin failed"
-	not <$> git_annex "drop" [annexedfile] @? "drop wrongly succeeded with no known copy of file"
+	git_annex_shouldfail "drop" [annexedfile] @? "drop wrongly succeeded with no known copy of file"
 	annexed_present annexedfile
 	git_annex "drop" ["--force", annexedfile] @? "drop --force failed"
 	annexed_notpresent annexedfile
@@ -450,7 +447,7 @@ test_drop_withremote = intmpclonerepo $ do
 	git_annex "get" [annexedfile] @? "get failed"
 	annexed_present annexedfile
 	git_annex "numcopies" ["2"] @? "numcopies config failed"
-	not <$> git_annex "drop" [annexedfile] @? "drop succeeded although numcopies is not satisfied"
+	git_annex_shouldfail "drop" [annexedfile] @? "drop succeeded although numcopies is not satisfied"
 	git_annex "numcopies" ["1"] @? "numcopies config failed"
 	git_annex "drop" [annexedfile] @? "drop failed though origin has copy"
 	annexed_notpresent annexedfile
@@ -464,7 +461,7 @@ test_drop_untrustedremote = intmpclonerepo $ do
 	git_annex "untrust" ["origin"] @? "untrust of origin failed"
 	git_annex "get" [annexedfile] @? "get failed"
 	annexed_present annexedfile
-	not <$> git_annex "drop" [annexedfile] @? "drop wrongly succeeded with only an untrusted copy of the file"
+	git_annex_shouldfail "drop" [annexedfile] @? "drop wrongly succeeded with only an untrusted copy of the file"
 	annexed_present annexedfile
 	inmainrepo $ annexed_present annexedfile
 
@@ -595,18 +592,18 @@ test_lock = intmpclonerepoInDirect $ do
 	annexed_notpresent annexedfile
 	unlessM (annexeval Annex.Version.versionSupportsUnlockedPointers) $
 		ifM (unlockedFiles <$> getTestMode)
-			( not <$> git_annex "lock" [annexedfile] @? "lock failed to fail with not present file"
-			, not <$> git_annex "unlock" [annexedfile] @? "unlock failed to fail with not present file"
+			( git_annex_shouldfail "lock" [annexedfile] @? "lock failed to fail with not present file"
+			, git_annex_shouldfail "unlock" [annexedfile] @? "unlock failed to fail with not present file"
 			)
 	annexed_notpresent annexedfile
 
 	-- regression test: unlock of newly added, not committed file
-	-- should fail in v5 mode. In v6 mode, this is allowed.
-	writeFile "newfile" "foo"
+	-- should fail in v5 mode. In v7 mode, this is allowed.
+	writecontent "newfile" "foo"
 	git_annex "add" ["newfile"] @? "add new file failed"
 	ifM (annexeval Annex.Version.versionSupportsUnlockedPointers)
-		( git_annex "unlock" ["newfile"] @? "unlock failed on newly added, never committed file in v6 repository"
-		, not <$> git_annex "unlock" ["newfile"] @? "unlock failed to fail on newly added, never committed file in v5 repository"
+		( git_annex "unlock" ["newfile"] @? "unlock failed on newly added, never committed file in v7 repository"
+		, git_annex_shouldfail "unlock" ["newfile"] @? "unlock failed to fail on newly added, never committed file in v5 repository"
 		)
 
 	git_annex "get" [annexedfile] @? "get of file failed"
@@ -616,10 +613,10 @@ test_lock = intmpclonerepoInDirect $ do
 	-- write different content, to verify that lock
 	-- throws it away
 	changecontent annexedfile
-	writeFile annexedfile $ content annexedfile ++ "foo"
-	not <$> git_annex "lock" [annexedfile] @? "lock failed to fail without --force"
+	writecontent annexedfile $ content annexedfile ++ "foo"
+	git_annex_shouldfail "lock" [annexedfile] @? "lock failed to fail without --force"
 	git_annex "lock" ["--force", annexedfile] @? "lock --force failed"
-	-- In v6 mode, the original content of the file is not always
+	-- In v7 mode, the original content of the file is not always
 	-- preserved after modification, so re-get it.
 	git_annex "get" [annexedfile] @? "get of file failed after lock --force"
 	annexed_present_locked annexedfile
@@ -642,19 +639,20 @@ test_lock = intmpclonerepoInDirect $ do
 -- Regression test: lock --force when work tree file
 -- was modified lost the (unmodified) annex object.
 -- (Only occurred when the keys database was out of sync.)
-test_lock_v6_force :: Assertion
-test_lock_v6_force = intmpclonerepoInDirect $ do
+test_lock_v7_force :: Assertion
+test_lock_v7_force = intmpclonerepoInDirect $ do
 	git_annex "upgrade" [] @? "upgrade failed"
 	whenM (annexeval Annex.Version.versionSupportsUnlockedPointers) $ do
 		git_annex "get" [annexedfile] @? "get of file failed"
-		git_annex "unlock" [annexedfile] @? "unlock failed in v6 mode"
+		git_annex "unlock" [annexedfile] @? "unlock failed in v7 mode"
 		annexeval $ do
+			Just k <- Annex.WorkTree.lookupFile annexedfile
+			Database.Keys.removeInodeCaches k
 			Database.Keys.closeDb
-			dbdir <- Annex.fromRepo Annex.Locations.gitAnnexKeysDb
-			liftIO $ renameDirectory dbdir (dbdir ++ ".old")
-		writeFile annexedfile "test_lock_v6_force content"
-		not <$> git_annex "lock" [annexedfile] @? "lock of modified file failed to fail in v6 mode"
-		git_annex "lock" ["--force", annexedfile] @? "lock --force of modified file failed in v6 mode"
+			liftIO . nukeFile =<< Annex.fromRepo Annex.Locations.gitAnnexKeysDbIndexCache
+		writecontent annexedfile "test_lock_v7_force content"
+		git_annex_shouldfail "lock" [annexedfile] @? "lock of modified file failed to fail in v7 mode"
+		git_annex "lock" ["--force", annexedfile] @? "lock --force of modified file failed in v7 mode"
 		annexed_present_locked annexedfile
 
 test_edit :: Assertion
@@ -683,7 +681,7 @@ test_edit' precommit = intmpclonerepoInDirect $ do
 		)
 	c <- readFile annexedfile
 	assertEqual "content of modified file" c (changedcontent annexedfile)
-	not <$> git_annex "drop" [annexedfile] @? "drop wrongly succeeded with no known copy of modified file"
+	git_annex_shouldfail "drop" [annexedfile] @? "drop wrongly succeeded with no known copy of modified file"
 
 test_partial_commit :: Assertion
 test_partial_commit = intmpclonerepoInDirect $ do
@@ -693,7 +691,7 @@ test_partial_commit = intmpclonerepoInDirect $ do
 	changecontent annexedfile
 	ifM (annexeval Annex.Version.versionSupportsUnlockedPointers)
 		( boolSystem "git" [Param "commit", Param "-q", Param "-m", Param "test", File annexedfile]
-			@? "partial commit of unlocked file should be allowed in v6 repository"
+			@? "partial commit of unlocked file should be allowed in v7 repository"
 		, not <$> boolSystem "git" [Param "commit", Param "-q", Param "-m", Param "test", File annexedfile]
 			@? "partial commit of unlocked file not blocked by pre-commit hook"
 		)
@@ -723,7 +721,7 @@ test_direct = intmpclonerepoInDirect $ do
 	git_annex "get" [annexedfile] @? "get of file failed"
 	annexed_present annexedfile
 	ifM (annexeval Annex.Version.versionSupportsUnlockedPointers)
-		( not <$> git_annex "direct" [] @? "switch to direct mode failed to fail in v6 repository"
+		( git_annex_shouldfail "direct" [] @? "switch to direct mode failed to fail in v7 repository"
 		, do
 			git_annex "direct" [] @? "switch to direct mode failed"
 			annexed_present annexedfile
@@ -769,10 +767,10 @@ test_fsck_basic = intmpclonerepo $ do
 	corrupt f = do
 		git_annex "get" [f] @? "get of file failed"
 		Utility.FileMode.allowWrite f
-		writeFile f (changedcontent f)
+		writecontent f (changedcontent f)
 		ifM (annexeval Config.isDirect <||> unlockedFiles <$> getTestMode)
 			( git_annex "fsck" [] @? "fsck failed on unlocked file with changed file content"
-			, not <$> git_annex "fsck" [] @? "fsck failed to fail with corrupted file content"
+			, git_annex_shouldfail "fsck" [] @? "fsck failed to fail with corrupted file content"
 			)
 		git_annex "fsck" [] @? "fsck unexpectedly failed again; previous one did not fix problem with " ++ f
 
@@ -803,7 +801,7 @@ test_fsck_fromremote = intmpclonerepo $ do
 	git_annex "fsck" ["--from", "origin"] @? "fsck --from origin failed"
 
 fsck_should_fail :: String -> Assertion
-fsck_should_fail m = not <$> git_annex "fsck" []
+fsck_should_fail m = git_annex_shouldfail "fsck" []
 	@? "fsck failed to fail with " ++ m
 
 test_migrate :: Assertion
@@ -875,10 +873,10 @@ test_unused = intmpclonerepoInDirect $ do
 		@? "dropkey failed"
 	checkunused [sha1annexedfilekey] ("after dropkey --force " ++ Key.key2file annexedfilekey)
 
-	not <$> git_annex "dropunused" ["1"] @? "dropunused failed to fail without --force"
+	git_annex_shouldfail "dropunused" ["1"] @? "dropunused failed to fail without --force"
 	git_annex "dropunused" ["--force", "1"] @? "dropunused failed"
 	checkunused [] "after dropunused"
-	not <$> git_annex "dropunused" ["--force", "10", "501"] @? "dropunused failed to fail on bogus numbers"
+	git_annex_shouldfail "dropunused" ["--force", "10", "501"] @? "dropunused failed to fail on bogus numbers"
 
 	-- Unused used to miss renamed symlinks that were not staged
 	-- and pointed at annexed content, and think that content was unused.
@@ -886,7 +884,7 @@ test_unused = intmpclonerepoInDirect $ do
 	-- unlocked, the work tree file has the content, and there's no way
 	-- to associate it with the key.
 	unlessM (unlockedFiles <$> getTestMode) $ do
-		writeFile "unusedfile" "unusedcontent"
+		writecontent "unusedfile" "unusedcontent"
 		git_annex "add" ["unusedfile"] @? "add of unusedfile failed"
 		unusedfilekey <- getKey backendSHA256E "unusedfile"
 		renameFile "unusedfile" "unusedunstagedfile"
@@ -897,7 +895,7 @@ test_unused = intmpclonerepoInDirect $ do
 
 	-- unused used to miss symlinks that were deleted or modified
 	-- manually
-	writeFile "unusedfile" "unusedcontent"
+	writecontent "unusedfile" "unusedcontent"
 	git_annex "add" ["unusedfile"] @? "add of unusedfile failed"
 	boolSystem "git" [Param "add", File "unusedfile"] @? "git add failed"
 	unusedfilekey' <- getKey backendSHA256E "unusedfile"
@@ -907,7 +905,7 @@ test_unused = intmpclonerepoInDirect $ do
 
 	-- unused used to false positive on symlinks that were
 	-- deleted or modified manually, but not staged as such
-	writeFile "unusedfile" "unusedcontent"
+	writecontent "unusedfile" "unusedcontent"
 	git_annex "add" ["unusedfile"] @? "add of unusedfile failed"
 	boolSystem "git" [Param "add", File "unusedfile"] @? "git add failed"
 	checkunused [] "with staged file"
@@ -919,10 +917,10 @@ test_unused = intmpclonerepoInDirect $ do
 	-- found as unused.
 	whenM (unlockedFiles <$> getTestMode) $ do
 		let f = "unlockedfile"
-		writeFile f "unlockedcontent1"
+		writecontent f "unlockedcontent1"
 		boolSystem "git" [Param "add", File "unlockedfile"] @? "git add failed"
 		checkunused [] "with unlocked file before modification"
-		writeFile f "unlockedcontent2"
+		writecontent f "unlockedcontent2"
 		checkunused [] "with unlocked file after modification"
 		not <$> boolSystem "git" [Param "diff", Param "--quiet", File f] @? "git diff did not show changes to unlocked file"
 		-- still nothing unused because one version is in the index
@@ -959,7 +957,7 @@ test_find = intmpclonerepo $ do
 	{- --include=* should match files in subdirectories too,
 	 - and --exclude=* should exclude them. -}
 	createDirectory "dir"
-	writeFile "dir/subfile" "subfile"
+	writecontent "dir/subfile" "subfile"
 	git_annex "add" ["dir"] @? "add of subdir failed"
 	git_annex_expectoutput "find" ["--include", "*", "--exclude", annexedfile, "--exclude", sha1annexedfile] ["dir/subfile"]
 	git_annex_expectoutput "find" ["--exclude", "*"] []
@@ -1043,10 +1041,10 @@ test_conflict_resolution_movein_regression = withtmpclonerepo $ \r1 ->
 			{- Set up a conflict. -}
 			let newcontent = content annexedfile ++ rname r
 			ifM (annexeval Config.isDirect)
-				( writeFile annexedfile newcontent
+				( writecontent annexedfile newcontent
 				, do
 					git_annex "unlock" [annexedfile] @? "unlock failed"		
-					writeFile annexedfile newcontent
+					writecontent annexedfile newcontent
 				)
 		{- Sync twice in r1 so it gets the conflict resolution
 		 - update from r2 -}
@@ -1070,12 +1068,12 @@ test_conflict_resolution =
 		withtmpclonerepo $ \r2 -> do
 			indir r1 $ do
 				disconnectOrigin
-				writeFile conflictor "conflictor1"
+				writecontent conflictor "conflictor1"
 				add_annex conflictor @? "add conflicter failed"
 				git_annex "sync" [] @? "sync failed in r1"
 			indir r2 $ do
 				disconnectOrigin
-				writeFile conflictor "conflictor2"
+				writecontent conflictor "conflictor2"
 				add_annex conflictor @? "add conflicter failed"
 				git_annex "sync" [] @? "sync failed in r2"
 			pair r1 r2
@@ -1098,20 +1096,20 @@ test_conflict_resolution =
 
 {- Conflict resolution while in an adjusted branch. -}
 test_conflict_resolution_adjusted_branch :: Assertion
-test_conflict_resolution_adjusted_branch = whenM Annex.AdjustedBranch.isGitVersionSupported $
+test_conflict_resolution_adjusted_branch =
 	withtmpclonerepo $ \r1 ->
-		withtmpclonerepo $ \r2 -> do
+		withtmpclonerepo $ \r2 -> whenM (adjustedbranchsupported r2) $ do
 			indir r1 $ do
 				disconnectOrigin
-				writeFile conflictor "conflictor1"
+				writecontent conflictor "conflictor1"
 				add_annex conflictor @? "add conflicter failed"
 				git_annex "sync" [] @? "sync failed in r1"
 			indir r2 $ do
 				disconnectOrigin
-				writeFile conflictor "conflictor2"
+				writecontent conflictor "conflictor2"
 				add_annex conflictor @? "add conflicter failed"
 				git_annex "sync" [] @? "sync failed in r2"
-				-- need v6 to use adjust
+				-- need v7 to use adjust
 				git_annex "upgrade" [] @? "upgrade failed"
 				-- We might be in an adjusted branch
 				-- already, when eg on a crippled
@@ -1146,13 +1144,13 @@ test_mixed_conflict_resolution = do
 		withtmpclonerepo $ \r2 -> do
 			indir r1 $ do
 				disconnectOrigin
-				writeFile conflictor "conflictor"
+				writecontent conflictor "conflictor"
 				add_annex conflictor @? "add conflicter failed"
 				git_annex "sync" [] @? "sync failed in r1"
 			indir r2 $ do
 				disconnectOrigin
 				createDirectory conflictor
-				writeFile subfile "subfile"
+				writecontent subfile "subfile"
 				add_annex conflictor @? "add conflicter failed"
 				git_annex "sync" [] @? "sync failed in r2"
 			pair r1 r2
@@ -1188,7 +1186,7 @@ test_remove_conflict_resolution = do
 		withtmpclonerepo $ \r2 -> do
 			indir r1 $ do
 				disconnectOrigin
-				writeFile conflictor "conflictor"
+				writecontent conflictor "conflictor"
 				add_annex conflictor @? "add conflicter failed"
 				git_annex "sync" [] @? "sync failed in r1"
 			indir r2 $
@@ -1201,7 +1199,7 @@ test_remove_conflict_resolution = do
 				unlessM (annexeval Config.isDirect) $ do
 					git_annex "unlock" [conflictor]
 						@? "unlock conflictor failed"
-				writeFile conflictor "newconflictor"
+				writecontent conflictor "newconflictor"
 			indir r1 $
 				nukeFile conflictor
 			let l = if inr1 then [r1, r2, r1] else [r2, r1, r2]
@@ -1237,12 +1235,12 @@ test_nonannexed_file_conflict_resolution = do
 			whenM (isInDirect r1 <&&> isInDirect r2) $ do
 				indir r1 $ do
 					disconnectOrigin
-					writeFile conflictor "conflictor"
+					writecontent conflictor "conflictor"
 					add_annex conflictor @? "add conflicter failed"
 					git_annex "sync" [] @? "sync failed in r1"
 				indir r2 $ do
 					disconnectOrigin
-					writeFile conflictor nonannexed_content
+					writecontent conflictor nonannexed_content
 					boolSystem "git"
 						[ Param "config"
 						, Param "annex.largefiles"
@@ -1294,7 +1292,7 @@ test_nonannexed_symlink_conflict_resolution = do
 			       <&&> isInDirect r1 <&&> isInDirect r2) $ do
 				indir r1 $ do
 					disconnectOrigin
-					writeFile conflictor "conflictor"
+					writecontent conflictor "conflictor"
 					add_annex conflictor @? "add conflicter failed"
 					git_annex "sync" [] @? "sync failed in r1"
 				indir r2 $ do
@@ -1345,12 +1343,12 @@ test_uncommitted_conflict_resolution = do
 			indir r1 $ do
 				disconnectOrigin
 				createDirectoryIfMissing True (parentDir remoteconflictor)
-				writeFile remoteconflictor annexedcontent
+				writecontent remoteconflictor annexedcontent
 				add_annex conflictor @? "add remoteconflicter failed"
 				git_annex "sync" [] @? "sync failed in r1"
 			indir r2 $ do
 				disconnectOrigin
-				writeFile conflictor localcontent
+				writecontent conflictor localcontent
 			pair r1 r2
 			indir r2 $ ifM (annexeval Config.isDirect)
 				( do
@@ -1366,7 +1364,7 @@ test_uncommitted_conflict_resolution = do
 				-- this case is intentionally not handled
 				-- in indirect mode, since the user
 				-- can recover on their own easily
-				, not <$> git_annex "sync" [] @? "sync failed to fail"
+				, git_annex_shouldfail "sync" [] @? "sync failed to fail"
 				)
 	conflictor = "conflictor"
 	localprefix = ".variant-local"
@@ -1382,18 +1380,18 @@ test_conflict_resolution_symlink_bit = unlessM (unlockedFiles <$> getTestMode) $
 		withtmpclonerepo $ \r2 ->
 			withtmpclonerepo $ \r3 -> do
 				indir r1 $ do
-					writeFile conflictor "conflictor"
+					writecontent conflictor "conflictor"
 					git_annex "add" [conflictor] @? "add conflicter failed"
 					git_annex "sync" [] @? "sync failed in r1"
 					check_is_link conflictor "r1"
 				indir r2 $ do
 					createDirectory conflictor
-					writeFile (conflictor </> "subfile") "subfile"
+					writecontent (conflictor </> "subfile") "subfile"
 					git_annex "add" [conflictor] @? "add conflicter failed"
 					git_annex "sync" [] @? "sync failed in r2"
 					check_is_link (conflictor </> "subfile") "r2"
 				indir r3 $ do
-					writeFile conflictor "conflictor"
+					writecontent conflictor "conflictor"
 					git_annex "add" [conflictor] @? "add conflicter failed"
 					git_annex "sync" [] @? "sync failed in r1"
 					check_is_link (conflictor </> "subfile") "r3"
@@ -1405,7 +1403,7 @@ test_conflict_resolution_symlink_bit = unlessM (unlockedFiles <$> getTestMode) $
 		all (\i -> Git.Types.toTreeItemType (Git.LsTree.mode i) == Just Git.Types.TreeSymlink) l
 			@? (what ++ " " ++ f ++ " lost symlink bit after merge: " ++ show l)
 
-{- A v6 unlocked file that conflicts with a locked file should be resolved
+{- A v7 unlocked file that conflicts with a locked file should be resolved
  - in favor of the unlocked file, with no variant files, as long as they
  - both point to the same key. -}
 test_mixed_lock_conflict_resolution :: Assertion
@@ -1414,12 +1412,12 @@ test_mixed_lock_conflict_resolution =
 		withtmpclonerepo $ \r2 -> do
 			indir r1 $ whenM shouldtest $ do
 				disconnectOrigin
-				writeFile conflictor "conflictor"
+				writecontent conflictor "conflictor"
 				git_annex "add" [conflictor] @? "add conflicter failed"
 				git_annex "sync" [] @? "sync failed in r1"
 			indir r2 $ whenM shouldtest $ do
 				disconnectOrigin
-				writeFile conflictor "conflictor"
+				writecontent conflictor "conflictor"
 				git_annex "add" [conflictor] @? "add conflicter failed"
 				git_annex "unlock" [conflictor] @? "unlock conflicter failed"
 				git_annex "sync" [] @? "sync failed in r2"
@@ -1447,9 +1445,9 @@ test_mixed_lock_conflict_resolution =
  - where the same file is added to both independently. The bad merge
  - emptied the whole tree. -}
 test_adjusted_branch_merge_regression :: Assertion
-test_adjusted_branch_merge_regression = whenM Annex.AdjustedBranch.isGitVersionSupported $
+test_adjusted_branch_merge_regression = do
 	withtmpclonerepo $ \r1 ->
-		withtmpclonerepo $ \r2 -> do
+		withtmpclonerepo $ \r2 -> whenM (adjustedbranchsupported r1) $ do
 			pair r1 r2
 			setup r1
 			setup r2
@@ -1461,7 +1459,7 @@ test_adjusted_branch_merge_regression = whenM Annex.AdjustedBranch.isGitVersionS
 		disconnectOrigin
 		git_annex "upgrade" [] @? "upgrade failed"
 		git_annex "adjust" ["--unlock", "--force"] @? "adjust failed"
-		writeFile conflictor "conflictor"
+		writecontent conflictor "conflictor"
 		git_annex "add" [conflictor] @? "add conflicter failed"
 		git_annex "sync" [] @? "sync failed"
 	checkmerge what d = indir d $ do
@@ -1474,22 +1472,21 @@ test_adjusted_branch_merge_regression = whenM Annex.AdjustedBranch.isGitVersionS
  - a subtree to an existing tree lost files. -}
 test_adjusted_branch_subtree_regression :: Assertion
 test_adjusted_branch_subtree_regression = 
-	whenM Annex.AdjustedBranch.isGitVersionSupported $ 
-		withtmpclonerepo $ \r -> do
-			indir r $ do
-				disconnectOrigin
-				git_annex "upgrade" [] @? "upgrade failed"
-				git_annex "adjust" ["--unlock", "--force"] @? "adjust failed"
-				createDirectoryIfMissing True "a/b/c"
-				writeFile "a/b/c/d" "foo"
-				git_annex "add" ["a/b/c"] @? "add a/b/c failed"
-				git_annex "sync" [] @? "sync failed"
-				createDirectoryIfMissing True "a/b/x"
-				writeFile "a/b/x/y" "foo"
-				git_annex "add" ["a/b/x"] @? "add a/b/x failed"
-				git_annex "sync" [] @? "sync failed"
-				boolSystem "git" [Param "checkout", Param "master"] @? "git checkout master failed"
-				doesFileExist "a/b/x/y" @? ("a/b/x/y missing from master after adjusted branch sync")
+	withtmpclonerepo $ \r -> whenM (adjustedbranchsupported r) $ do
+		indir r $ do
+			disconnectOrigin
+			git_annex "upgrade" [] @? "upgrade failed"
+			git_annex "adjust" ["--unlock", "--force"] @? "adjust failed"
+			createDirectoryIfMissing True "a/b/c"
+			writecontent "a/b/c/d" "foo"
+			git_annex "add" ["a/b/c"] @? "add a/b/c failed"
+			git_annex "sync" [] @? "sync failed"
+			createDirectoryIfMissing True "a/b/x"
+			writecontent "a/b/x/y" "foo"
+			git_annex "add" ["a/b/x"] @? "add a/b/x failed"
+			git_annex "sync" [] @? "sync failed"
+			boolSystem "git" [Param "checkout", Param "master"] @? "git checkout master failed"
+			doesFileExist "a/b/x/y" @? ("a/b/x/y missing from master after adjusted branch sync")
 
 {- Set up repos as remotes of each other. -}
 pair :: FilePath -> FilePath -> Assertion
@@ -1518,7 +1515,7 @@ test_uninit = intmpclonerepo $ do
 test_uninit_inbranch :: Assertion
 test_uninit_inbranch = intmpclonerepoInDirect $ do
 	boolSystem "git" [Param "checkout", Param "git-annex"] @? "git checkout git-annex"
-	not <$> git_annex "uninit" [] @? "uninit failed to fail when git-annex branch was checked out"
+	git_annex_shouldfail "uninit" [] @? "uninit failed to fail when git-annex branch was checked out"
 
 test_upgrade :: Assertion
 test_upgrade = intmpclonerepo $
@@ -1529,7 +1526,7 @@ test_whereis = intmpclonerepo $ do
 	annexed_notpresent annexedfile
 	git_annex "whereis" [annexedfile] @? "whereis on non-present file failed"
 	git_annex "untrust" ["origin"] @? "untrust failed"
-	not <$> git_annex "whereis" [annexedfile] @? "whereis on non-present file only present in untrusted repo failed to fail"
+	git_annex_shouldfail "whereis" [annexedfile] @? "whereis on non-present file only present in untrusted repo failed to fail"
 	git_annex "get" [annexedfile] @? "get failed"
 	annexed_present annexedfile
 	git_annex "whereis" [annexedfile] @? "whereis on present file failed"
@@ -1555,7 +1552,7 @@ test_hook_remote = intmpclonerepo $ do
 	annexed_notpresent annexedfile
 	git_annex "move" [annexedfile, "--from", "foo"] @? "move --from hook remote failed"
 	annexed_present annexedfile
-	not <$> git_annex "drop" [annexedfile, "--numcopies=2"] @? "drop failed to fail"
+	git_annex_shouldfail "drop" [annexedfile, "--numcopies=2"] @? "drop failed to fail"
 	annexed_present annexedfile
   where
 	dir = "dir"
@@ -1579,7 +1576,7 @@ test_directory_remote = intmpclonerepo $ do
 	annexed_notpresent annexedfile
 	git_annex "move" [annexedfile, "--from", "foo"] @? "move --from directory remote failed"
 	annexed_present annexedfile
-	not <$> git_annex "drop" [annexedfile, "--numcopies=2"] @? "drop failed to fail"
+	git_annex_shouldfail "drop" [annexedfile, "--numcopies=2"] @? "drop failed to fail"
 	annexed_present annexedfile
 
 test_rsync_remote :: Assertion
@@ -1594,7 +1591,7 @@ test_rsync_remote = intmpclonerepo $ do
 	annexed_notpresent annexedfile
 	git_annex "move" [annexedfile, "--from", "foo"] @? "move --from rsync remote failed"
 	annexed_present annexedfile
-	not <$> git_annex "drop" [annexedfile, "--numcopies=2"] @? "drop failed to fail"
+	git_annex_shouldfail "drop" [annexedfile, "--numcopies=2"] @? "drop failed to fail"
 	annexed_present annexedfile
 
 test_bup_remote :: Assertion
@@ -1658,7 +1655,7 @@ test_crypto = do
 			annexed_notpresent annexedfile
 			git_annex "move" [annexedfile, "--from", "foo"] @? "move --from encrypted remote failed"
 			annexed_present annexedfile
-			not <$> git_annex "drop" [annexedfile, "--numcopies=2"] @? "drop failed to fail"
+			git_annex_shouldfail "drop" [annexedfile, "--numcopies=2"] @? "drop failed to fail"
 			annexed_present annexedfile
 	{- Ensure the configuration complies with the encryption scheme, and
 	 - that all keys are encrypted properly for the given directory remote. -}
@@ -1695,7 +1692,7 @@ test_crypto = putStrLn "gpg testing not implemented on Windows"
 test_add_subdirs :: Assertion
 test_add_subdirs = intmpclonerepo $ do
 	createDirectory "dir"
-	writeFile ("dir" </> "foo") $ "dir/" ++ content annexedfile
+	writecontent ("dir" </> "foo") $ "dir/" ++ content annexedfile
 	git_annex "add" ["dir"] @? "add of subdir failed"
 
 	{- Regression test for Windows bug where symlinks were not
@@ -1707,7 +1704,7 @@ test_add_subdirs = intmpclonerepo $ do
 		"../.git/annex/" `isPrefixOf` l @? ("symlink from subdir to .git/annex is wrong: " ++ l)
 
 	createDirectory "dir2"
-	writeFile ("dir2" </> "foo") $ content annexedfile
+	writecontent ("dir2" </> "foo") $ content annexedfile
 	setCurrentDirectory "dir"
 	git_annex "add" [".." </> "dir2"] @? "add of ../subdir failed"
 
@@ -1717,8 +1714,8 @@ test_addurl = intmpclonerepo $ do
 	let filecmd c ps = git_annex c ("-cannex.security.allowed-url-schemes=file" : ps)
 	f <- absPath "myurl"
 	let url = replace "\\" "/" ("file:///" ++ dropDrive f)
-	writeFile f "foo"
-	not <$> git_annex "addurl" [url] @? "addurl failed to fail on file url"
+	writecontent f "foo"
+	git_annex_shouldfail "addurl" [url] @? "addurl failed to fail on file url"
 	filecmd "addurl" [url] @? ("addurl failed on " ++ url)
 	let dest = "addurlurldest"
 	filecmd "addurl" ["--file", dest, url] @? ("addurl failed on " ++ url ++ "  with --file")
