@@ -10,6 +10,7 @@ module Test.Framework where
 import Test.Tasty
 import Test.Tasty.Runners
 import Test.Tasty.HUnit
+import Control.Concurrent
 
 import Common
 import Types.Test
@@ -17,6 +18,7 @@ import Types.Test
 import qualified Annex
 import qualified Annex.UUID
 import qualified Annex.Version
+import qualified Types.RepoVersion
 import qualified Backend
 import qualified Git.CurrentRepo
 import qualified Git.Construct
@@ -33,6 +35,7 @@ import qualified Annex.Link
 import qualified Annex.Init
 import qualified Annex.Path
 import qualified Annex.Action
+import qualified Annex.AdjustedBranch
 import qualified Utility.Process
 import qualified Utility.Env
 import qualified Utility.Env.Set
@@ -45,14 +48,24 @@ import qualified CmdLine.GitAnnex as GitAnnex
 -- This is equivilant to running git-annex, but it's all run in-process
 -- so test coverage collection works.
 git_annex :: String -> [String] -> IO Bool
-git_annex command params = do
+git_annex command params = git_annex' command params >>= \case
+	Right () -> return True
+	Left e -> do
+		hPutStrLn stderr (show e)
+		return False
+
+-- For when git-annex is expected to fail.
+git_annex_shouldfail :: String -> [String] -> IO Bool
+git_annex_shouldfail command params = git_annex' command params >>= \case
+	Right () -> return False
+	Left _ -> return True
+
+git_annex' :: String -> [String] -> IO (Either SomeException ())
+git_annex' command params = do
 	-- catch all errors, including normally fatal errors
-	r <- try run ::IO (Either SomeException ())
-	case r of
-		Right _ -> return True
-		Left _ -> return False
+	try run ::IO (Either SomeException ())
   where
-	run = GitAnnex.run dummyTestOptParser Nothing (command:"-q":params)
+	run = GitAnnex.run dummyTestOptParser (\_ -> noop) (command:"-q":params)
 	dummyTestOptParser = pure mempty
 
 {- Runs git-annex and returns its output. -}
@@ -148,18 +161,23 @@ disconnectOrigin = boolSystem "git" [Param "remote", Param "rm", Param "origin"]
 withgitrepo :: (FilePath -> Assertion) -> Assertion
 withgitrepo = bracket (setuprepo mainrepodir) return
 
-indir :: FilePath -> Assertion -> Assertion
+indir :: FilePath -> IO a -> IO a
 indir dir a = do
 	currdir <- getCurrentDirectory
 	-- Assertion failures throw non-IO errors; catch
 	-- any type of error and change back to currdir before
 	-- rethrowing.
-	r <- bracket_ (changeToTmpDir dir) (setCurrentDirectory currdir)
-		(try a::IO (Either SomeException ()))
+	r <- bracket_
+		(changeToTmpDir dir)
+		(setCurrentDirectory currdir)
+		(tryNonAsync a)
 	case r of
-		Right () -> return ()
+		Right v -> return v
 		Left e -> throwM e
 
+adjustedbranchsupported :: FilePath -> IO Bool
+adjustedbranchsupported repo = indir repo $ annexeval Annex.AdjustedBranch.isSupported
+
 setuprepo :: FilePath -> IO FilePath
 setuprepo dir = do
 	cleanup dir
@@ -198,7 +216,7 @@ clonerepo old new cfg = do
 		ver <- annexVersion <$> getTestMode
 		if ver == Annex.Version.defaultVersion
 			then git_annex "init" ["-q", new] @? "git annex init failed"
-			else git_annex "init" ["-q", new, "--version", ver] @? "git annex init failed"
+			else git_annex "init" ["-q", new, "--version", show (Types.RepoVersion.fromRepoVersion ver)] @? "git annex init failed"
 	unless (bareClone cfg) $
 		indir new $
 			setupTestMode
@@ -387,11 +405,11 @@ add_annex f = ifM (unlockedFiles <$> getTestMode)
 data TestMode = TestMode
 	{ forceDirect :: Bool
 	, unlockedFiles :: Bool
-	, annexVersion :: Annex.Version.Version
+	, annexVersion :: Types.RepoVersion.RepoVersion
 	, keepFailures :: Bool
 	} deriving (Read, Show)
 
-testMode :: TestOptions -> Annex.Version.Version -> TestMode
+testMode :: TestOptions -> Types.RepoVersion.RepoVersion -> TestMode
 testMode opts v = TestMode
 	{ forceDirect = False
 	, unlockedFiles = False
@@ -502,8 +520,29 @@ content f
 	| "import" `isPrefixOf` f = "imported content"
 	| otherwise = "unknown file " ++ f
 
+-- Writes new content to a file, and makes sure that it has a different
+-- mtime than it did before
+writecontent :: FilePath -> String -> IO ()
+writecontent f c = go (10000000 :: Integer)
+  where
+	go ticsleft = do
+		oldmtime <- catchMaybeIO $ getModificationTime f
+		writeFile f c
+		newmtime <- getModificationTime f
+		if Just newmtime == oldmtime
+			then do
+				threadDelay 100000
+				let ticsleft' = ticsleft - 100000
+				if ticsleft' > 0
+					then go ticsleft'
+					else do
+						hPutStrLn stderr "file mtimes do not seem to be changing (tried for 10 seconds)"
+						hFlush stderr
+						return ()
+			else return ()
+
 changecontent :: FilePath -> IO ()
-changecontent f = writeFile f $ changedcontent f
+changecontent f = writecontent f $ changedcontent f
 
 changedcontent :: FilePath -> String
 changedcontent f = content f ++ " (modified)"
